//#define LineTest
//#define IRTest
//#define SPITESTHARNESS
//#define SPIBEN

#define RECEIVE_BUFFER_FULL (SPISR & _S12_SPIF)
#define TRANSMIT_BUFFER_EMPTY (SPISR & _S12_SPTEF)

#include <hidef.h>      /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <stdio.h>
#include <ADS12.h> 
#include <stdlib.h>
#include <S12E128bits.h> /* E128 bit definitions  */
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */
#include <stdlib.h>

//Custom module includes
#include "Timers.h"
#include "PWM.h"
#include "InitPorts.h"
#include "SPI.h"
#include "DeadReckoning.h"
#include "Movement.h"
#include "Sensors.h"
#include <string.h>
#include <ctype.h>


#pragma LINK_INFO DERIVATIVE "SampleS12"


char PerformingTasks(char state);
char PerformingIRTasks(char state);






#ifdef IRTest
void main(void) {
  
  char state = 0;
  
  InitPorts();
  InitPWM();
  InitTimers();
  
  (void)printf("Starting in state %i\n\r", state);
  while(1)
  {
    state = PerformingIRTasks(state);
  }
}
#endif

char PerformingIRTasks(char state)
{
  char Status =1;
  static char count;
  
  switch (state)
  {
    case 0:
      //Back up
      LeftSpeed(90);
      RightSpeed(100);
    
      if((TIM0_TFLG1 & _S12_C4F) != 0 )
      {
        (void)printf("Reseting timer, Count=%i\n\r", count);
        SetTimer(200);  //time in ms
        count += 1;
        if(count > 10)
        {
          Status = 0;
        } 
      }
    case 1:
      //Rotating while seeking IR
      RightSpeed(100);
      LeftSpeed(-97);
      if(CheckIR() == 1)
        state = 0;
      break;
    case 2:
      //Rotating while seeking IR
      RightSpeed(100);
      LeftSpeed(97);
      if(CheckIR() == 0)
        state = 1;
      break;
  }
  (void)printf("IR: %i      State: %i\n\r", CheckIR(), state);
  return state;
}

///////////////////////////////////////////////////////////
//Satisfies checkpoint (adjust 
#ifdef LineTest
void main(void) {
  
  char state = 1;
  
  InitPorts();
  InitPWM();
  InitTimers();
  RightSpeed(100);
  LeftSpeed(100);
  
  (void)printf("Starting in state %i\n\r", state);
  while(1)
  {
    state = PerformingTasks(state);
  }
}
#endif


char PerformingTasks(char state)
{
  static char count=0;
  char Status = 1;
  switch (state)
  {
    
  case 1:
    //Going Forward on line (while checking for alignment)
    Status = DrivingTape(0);
    break;
  case 2:  //Rotate far enough that we're clear of crossing strip
    //Set a timer and start backing up
    
    //Back up
    LeftSpeed(-100);
    RightSpeed(100);
    
    if((TIM0_TFLG1 & _S12_C4F) != 0 )
    {
      (void)printf("Reseting timer, Count=%i\n\r", count);
      SetTimer(200);  //time in ms
      count += 1;
      if(count > 10)
      {
        Status = 0;
      }
    }

    break;
  case 3:
    //Turning around while probing for sensor
    if(CheckTape(8) == 0)  //Still haven't reached tape
    {
    (void)printf("Rotating toward Tape");
      LeftSpeed(-100);
      RightSpeed(100);
      Status = 1;
    }
    else
    {
    (void)printf("Found Tape");
      LeftSpeed(0);
      RightSpeed(0);
      Status = 0;
    }
    break;
  case 4:
    //Going Forward on line (while checking for alignment)
    Status = DrivingTape(0); 
    break;
  case 5:
    //Stop
    LeftSpeed(0);
    RightSpeed(0);
    Status = 1;
    break;
  default:
    printf("Error: Invalid state %i\n\r", state);
  }
    
(void)printf(" Value:  L:%i  C:%i  R:%i\r", CheckTape(7), CheckTape(8), CheckTape(9));  

    
  if(Status == 0)
  {
    state += 1;
    (void)printf("\nState is %i\n", state);
  }
  return state;
}





#ifdef SPIBEN
void main()
{
    SPIM_InitializeSPI();
    InitTimers();
    while(1)
    {
        unsigned char Response, CommandRequest1, CommandRequest2;
        unsigned char CommandRequest;
        unsigned char num;
        
        (void)printf("Enter command: \n\r");
        CommandRequest1 = ((unsigned char)getchar());
        switch (CommandRequest1)
        {
          case 'a':
            Response = SPIM_TransferByte(0xE0);
            ResponseDecoder(Response);
           
          case 's':
           SPIDR = (0xE0);
           printf("Sent 0xE0\r\n");
           break;

          case 'r':
           printf(RECEIVE_BUFFER_FULL?("\r\nReceive buffer is full\r\n"):("Receive buffer is empty\r\n"));
           break;
          case 't':
           printf(TRANSMIT_BUFFER_EMPTY?("\r\nTransmit buffer is empty\r\n"):("Transmit buffer is full\r\n"));
           break;

          case 'x':
           printf("\r\nThe Target Commander says: %X\r\n",SPIDR);
           break;
        }
        (void)printf("\n\r");
    }
}
#endif

#ifdef SPITESTHARNESS
void main(void) 
{
    
    SPIM_InitializeSPI();
    InitTimers();
    
    while(1)
    {
        unsigned char Response, CommandRequest1, CommandRequest2;
        unsigned char CommandRequest;
        unsigned char num;
        
            
           //This works
            (void)printf("Enter 1st hex bit: \n\r");
            CommandRequest1 = toupper((unsigned char)getchar());
            switch (CommandRequest1)
            {
              case ('K'):
                Response = SPI_Transmit_And_Receive2(0xAA);
                while ( ResponseDecoder(Response) != EV_RECIEVED_TARGET)
                {
                  Response = SPI_Transmit_And_Receive2(0xAA);
                }
                num = (unsigned int)(Response & 0b00001111);
                printf("num is %i\r\n", num);
                Request_IR_Illumination(num);
                break;
               
              case ('R'):
                Request_Beacon_Status(15);
                break;
                
              case ('I'):
                Request_IR_Illumination(15);
                break;
                
              case ('A'):
                CommandRequest = 0xAA;
                (void)printf("Command is %x\r\n", CommandRequest);
                break;
                
              case ('E'):
                (void)printf("Enter your Second(E): \r\n");
                num = toupper((unsigned char)getchar());

                switch (num)
                { 
                  case ('0'):
                    CommandRequest = 0xE0;
                    break;
                  case ('1'):
                    CommandRequest = 0xE1;
                    break;
                  case ('2'):
                    CommandRequest = 0xE2;
                    break;
                  case ('3'):
                    CommandRequest = 0xE3;
                    break;
                  case ('4'):
                    CommandRequest = 0xE4;
                    break;
                  case ('5'):
                    CommandRequest = 0xE5;
                    break;
                  case ('6'):
                    CommandRequest = 0xE6;
                    break;
                  case ('7'):
                    CommandRequest = 0xE7;
                    break;
                  case ('8'):
                    CommandRequest = 0xE8;
                    break;
                  case ('9'):
                    CommandRequest = 0xE9;
                    break;
                  case ('A'):
                    CommandRequest = 0xEA;
                    break;
                  case ('B'):
                    CommandRequest = 0xEB;
                    break;
                  case ('C'):
                    CommandRequest = 0xEC;
                    break;
                  case ('D'):
                    CommandRequest = 0xED;
                    break;
                  case ('E'):
                    CommandRequest = 0xEE;
                    break;
                  case ('F'):
                    CommandRequest = 0xEF;
                    break;
                }
                (void)printf("Command is %x\r\n", CommandRequest);
                break;
                
              case ('B'):
                (void)printf("Enter your Second(B): \r\n");
                num = toupper((unsigned char)getchar());

                switch (num)
                { 
                  case ('0'):
                    CommandRequest = 0xB0;
                    break;
                  case ('1'):
                    CommandRequest = 0xB1;
                    break;
                  case ('2'):
                    CommandRequest = 0xB2;
                    break;
                  case ('3'):
                    CommandRequest = 0xB3;
                    break;
                  case ('4'):
                    CommandRequest = 0xB4;
                    break;
                  case ('5'):
                    CommandRequest = 0xB5;
                    break;
                  case ('6'):
                    CommandRequest = 0xB6;
                    break;
                  case ('7'):
                    CommandRequest = 0xB7;
                    break;
                  case ('8'):
                    CommandRequest = 0xB8;
                    break;
                  case ('9'):
                    CommandRequest = 0xB9;
                    break;
                  case ('A'):
                    CommandRequest = 0xBA;
                    break;
                  case ('B'):
                    CommandRequest = 0xBB;
                    break;
                  case ('C'):
                    CommandRequest = 0xBC;
                    break;
                  case ('D'):
                    CommandRequest = 0xBD;
                    break;
                  case ('E'):
                    CommandRequest = 0xBE;
                    break;
                  case ('F'):
                    CommandRequest = 0xBF;
                    break;
                }
                (void)printf("Command is %x\r\n", CommandRequest);
                break;
            }
            
            //Response = SPIM_TransferByte(CommandRequest);
            //Response = SPI_Transmit_And_Receive2(CommandRequest);
            //ResponseDecoder(Response);

            (void)printf("\r\n");
          
    }
}

#endif


#ifdef IRBEACON
void main()
{
  InitTimers();
  (void)ADS12_Init("AIIIIIII");
  PTIAD = BIT5HI;  // LED off
  PTP = BIT0HI;
  DDRP = DDRP | BIT0HI;  
  while(1)
  {
          if((PTIAD & BIT4HI) == 0)  // lo = true
          {
             PTP = BIT0LO;
             PTIAD = BIT5HI;  // LED off
             printf("5 is HI\r\n");
          }
          else
          {
            PTP = BIT0HI;
            PTIAD = BIT5LO;
            printf("5 is LO\r\n");
          }
          
  }
  
}
#endif


