/****************************************************************************
 Module
   CchEvents.c

 Revision
   1.0.1

 Description
   This is the Event Checking Module 

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------

****************************************************************************/
#include <stdio.h>
#include <ctype.h>
#include <hidef.h>         /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <S12E128bits.h> /* E128 bit definitions  */
#include "ChkEvents.h"
//#include "CommunicationsSM.h"
#include "SPI.h"
#include "SMEVENTS.h"
#include "Timers.h"
#include "PlayingSM.h"
#include "Search_And_DestroySM.h"
#include "FiringSM.h"
#include "ReturnHomeSM.h"
#include "Sensors.h"

#define _ms_ *187 

static int ActiveTargetArray[16] = 0;
static int BeaconStatusFlag = 0;
static int ActiveTarget;


//Main event checker (to be placed in main polling loop)
Event_t CheckEvents(void)
{
  char KeyStroke;
  Event_t CurrentEvent;
  Event_t Temporary = CheckSPI();


  //Test State machine with these manual (keyboard hit triggered) events
    if ( kbhit() != 0)       // there was a key pressed
    {
        KeyStroke = getchar();
        switch ( toupper(KeyStroke))
        {
            case '1':
              (void)printf(" Value:  %i  %i  %i  %i  %i  %i  %i\r\n", CheckTape(1), CheckTape(2), CheckTape(3), CheckTape(5), CheckTape(7), CheckTape(8), CheckTape(9));
              break;
            case 'A' :  
              CurrentEvent = EV_RECIEVED_TARGET; 
              (void)printf("Triggering Manually\n\r");
              ActiveTarget = 0;
            break;
            case 'B' : CurrentEvent = EV_BEACON_INNACTIVE; break;
            case 'C' : CurrentEvent = EV_BEACON_ACTIVE_NOT_ILLUM; break;
            case 'D' : CurrentEvent = EV_BEACON_ACTIVE_ILLUM; break;
            case 'E' : CurrentEvent = EV_BEACON_HIT; break;
            case 'F' : CurrentEvent = EV_GAME_NOT_STARTED; break;
            case 'G' : CurrentEvent = EV_RETURN_HOME; break;
            case 'H' : CurrentEvent = EV_GAME_OVER_RED_WINS; break;
            case 'I' : CurrentEvent = EV_GAME_OVER_GREEN_WINS; break;
            case 'J' : CurrentEvent = EV_OFFLINE; break;
            case 'K' : CurrentEvent = EV_COMMANDER_BUSY; break;
            case 'L' : CurrentEvent = EV_GAME_NOT_STARTED; break;
            case 'N' : CurrentEvent = EV_GAME_OVER_RED_WINS; break;
            
            case 'O' : CurrentEvent = EV_GAME_NOT_STARTED; break;
            case 'Q' : CurrentEvent = EV_GAME_OVER_RED_WINS; break;
            
        }  
        (void)printf("Received Keyboard input\r\n");
        return(CurrentEvent);
    }  
    





 
    if( Temporary != EV_NO_EVENT)
    {
      (void)printf("Active Target is: %i\n\r", GetTarget()); 
      //return EV_RECIEVED_TARGET;
      return Temporary;
    }
    

    if( QueryTapeFoundFlag() == 1)
    {
       (void)printf("TapeFound Event\r\n");  
       return EV_TAPE_FOUND;
    }
    
    
    if( QueryTapeAlignedFlag() == 1 )
    { 
      (void)printf("TapeAligned Event\r\n");  
      return EV_TAPE_ALIGNED;
    }
    
    
    if( QueryTIntersectFlag() == 1 )
    {
      (void)printf("TIntersect Event\r\n");
      return EV_TIntersect_FOUND;
    }
    
    if ( QueryAngleFlag() == 1)
    {
      (void)printf("AngleFlag Event\r\n");
      return EV_ROTATE_DONE;
    }
    
    if ( QueryFireFlag() == 1)
    {
      (void)printf("FireFlag is set in checkEvents\r\n");
      return EV_BALL_IS_RELEASED;
    } 

    if ( QueryAngleHomeFlag() == 1)
    {
      (void)printf("AngleFlagHome Event\r\n");
      return EV_ROTATE_HOME;
    }
    
    if( QueryReachHomeFlag() == 1 )
    {
      (void)printf("ReachHome Event\r\n");
      return EV_GOT_HOME;
    }
    
    return EV_NO_EVENT;
}




///////////////////////////////////////////////////////////////
//Query/Checking functions
///////////////////////////////////////////////////////////////


Event_t CheckSPI()
{
  unsigned char Response;
  static int OldNum=255;
  int num, BeaconStatusFlag;
  static Event_t OldEvent;
  Event_t NewEvent;
    //Response = SPI_Transmit_And_Receive2(0xAA);
    
    if ( SPI_TIMER_EXPIRE )
    {
    
        Response = SPI_Transmit_And_Receive2(0xAA);
        NewEvent = ResponseDecoder(Response);
        switch (NewEvent)
        {
          case    EV_RECIEVED_TARGET:
          //if (NewEvent == EV_RECIEVED_TARGET) 
          {
          
            num = (unsigned int)(Response & 0b00001111);
            //(void)printf("num is %i\r\n", num);
            ActiveTargetArray[num] = 1;
            ActiveTarget = num;
           
            BeaconStatusFlag = 1;  // Look for hit target
            
            if(OldNum != num)
            {
           
              TIM2_TC5 = TIM2_TCNT + (300 _ms_);
              TIM2_TFLG1 = _S12_C5F; //Clear the OC2 flag
              OldNum = num; 
              return NewEvent;
            }
          }
          break;
          
          case    EV_RETURN_HOME: 
          {
            if(NewEvent != OldEvent)
            {
              // Reset timer
              TIM2_TC5 = TIM2_TCNT + (300 _ms_);
              TIM2_TFLG1 = _S12_C5F; //Clear the OC2 flag
              OldEvent = NewEvent;
              
              //Target 16 is RED_BASE 
              if(GetSide() == 0)
              {
                ActiveTarget = 16;
              }
              else   //target 17 is GREEN_BASE
              { 
                ActiveTarget = 17;
              }
              
              (void)printf("Do we come here alot....in checkspi?");
              return NewEvent;
            }
          }
          break;
        }
        //Reset Timer
        TIM2_TC5 = TIM2_TCNT + (300 _ms_);  
        TIM2_TFLG1 = _S12_C5F; //Clear the OC2 flag
    } 
    
      return EV_NO_EVENT;
}

int GetTarget(void)
{
    return  ActiveTarget;
}


Event_t CheckTargetMissed(void)
{
    static int OldTarget;
    int NewTarget;
    NewTarget = GetTarget();
    if( NewTarget == OldTarget) 
    {
        OldTarget = NewTarget;
        return  EV_TARGET_MISSED;
    } 
    else 
    {
        OldTarget = NewTarget;
        return  EV_TARGET_HIT;
    }
}
  





    
