/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "CommunicationsSM.h"
#include "SPI_On_SM.h"

#include <hidef.h>      /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <stdio.h>
#include <ADS12.h> 
#include <stdlib.h>
#include <S12E128bits.h> /* E128 bit definitions  */
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */
#include <stdlib.h>



/*----------------------------- Module Defines ----------------------------*/
// define constants for the states for this machine
// and any other local defines

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/
static Event_t DuringSPI_On( Event_t Event);
static Event_t DuringSPI_Off( Event_t Event);



/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static Communications_State CurrentState;
// static char CurrentTarget;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM 
****************************************************************************/
Event_t RunCommunicationsSM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   Communications_State NextState = CurrentState;
   Event_t EntryEventKind = EV_ENTRY;

   switch ( CurrentState )
   {
       case SPI_ON:       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringSPI_On(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_GAME_OVER_GREEN_WINS : //If event is event one
                  // Execute action function for state one : event one
                  NextState = SPI_OFF;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //??????EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  //????CurrentEvent = NewEvent;
                  break;
               
               case EV_GAME_OVER_RED_WINS : //If event is event one
                  // Execute action function for state one : event one
                  NextState = SPI_OFF;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                
               case EV_RETURN_TO_BOX : //If event is event one
                  // Execute action function for state one : event one
                  NextState = SPI_OFF;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                  
            }
         }
         break;
         
       case SPI_OFF:       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringSPI_Off(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            
               // Not checking for any events. We want to be stuck in 
               // in SPI_OFF forever
            
         }
     
    }
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       (void)RunCommunicationsSM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       (void)RunCommunicationsSM(EntryEventKind);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartCommunicationsSM ( void )
{
   // to implement entry to a history state or directly to a substate
   // you can modify the initialization of the CurrentState variable
   // otherwise just start in the entry state every time the state machine
   // is started
   //if ( EV_ENTRY_HISTORY != CurrentEvent )
   //{
        CurrentState = SPI_ON;
   //}
   // call the entry function (if any) for the ENTRY_STATE
   (void)RunCommunicationsSM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
Communications_State QueryCommunicationsSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringSPI_On( Event_t Event)
{
   
    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if (Event == EV_ENTRY)
    {
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        (void)StartSPI_On_SM( Event );
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
        printf("In SPI_On State\n\r");
    }
    else if ( Event == EV_EXIT )
    {
        // on exit, give the lower levels a chance to clean up first
        (void)RunSPI_On_SM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        (void)RunSPI_On_SM(Event);
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}



static Event_t DuringSPI_Off( Event_t Event)
{
   
    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if (Event == EV_ENTRY)
    {
        (void)printf("In SPI_Off State\n\r");
        // Do Nothing
    }
    else if ( Event == EV_EXIT )
    {
        // Do Nothing
    }else
    // do the 'during' function for this state
    {
        // Do Nothing
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}
