//Defines physical constants

//All DISTANCE UNITS are in inches
//All ANGLE UNITS are in encoder ticks, which can be converted to more conventional units by
//TicksPerRotation...I would guess that 1 rotation ~ 5700ticks


#define PI 3.14159265358979323846

#define Sw (7.5)  //Space between the 2 drive wheels 7.5 (inches)
#define Dw (3)  //Diameter of wheel 76mm = 3 (inches)
#define Tw 5300  //Ticks per Rotation of wheel
//Note on DistancePertick mathematics:
//1.15 is an empirical derived constant
//factor of 2 was ommitted
//Gear ratio = 24/10 = 2.4
//If anyone wanted to check, I would guess that the Gear ratio was omitted, and we made a 'band-aid'
//by omitting the 2 *1.15 = 2.3
//However, the empirical constant is working, so I'm not changing it!!!
#define DistancePerTick 1.15 * ((2*PI*Dw)/(Tw))  //Distance center of bot (midpoint between wheels) travels per tick
#define TicksPerRotation (int)(Tw*((Sw)/Dw)/1.15)  //Ticks per rotation of the entire bot




//Distance from center of bot (pt. that dead reckoning considers) to gun...
//projected along axis perpendicular to the direction of fire ( used for asin correction term)
#define GUN_TO_CENTER 4.25

//Used to set a max duty cycle in code so that we don't damage the motors
#define FULL_DUTY 160

//Used to test current wheel directions (assuming wheels are going the direction that they are being driven)
#define LeftForward (PTP & BIT1HI)  
#define RightForward !(PTP & BIT2HI)  


#define DIST_FROM_WALL 2.625  //Distance from the field boundary to the target center
#define WALL_HEIGHT 97.0  //Y dimension of field
#define WALL_WIDTH 98.0  //X dimension of field

//Location of bot CENTER, when FRONT tape sensors detect the end of tape - used to initialize dead reckoning
//Assuming bot starts on green half of field
#define TAPE_T_GREEN_Y 24   
#define TAPE_T_GREEN_X 23.5
#define TAPE_T_GREEN_ANGLE (-TicksPerRotation/4)

//Location of bot CENTER, when FRONT tape sensors detect the end of tape - used to initialize dead reckoning
//Assuming bot starts on red half of field
#define TAPE_T_RED_Y (WALL_HEIGHT - 24)
#define TAPE_T_RED_X (WALL_WIDTH - 23.5)
#define TAPE_T_RED_ANGLE (TicksPerRotation/4)

//Location of red base (corner was used for return home function
//(this makes the bot hit the wall, and slide into correct position)
#define RED_BASE_Y 0.0
#define RED_BASE_X (WALL_WIDTH)

//Location of green base (corner was used for return home function
//(this makes the bot hit the wall, and slide into correct position)
#define GREEN_BASE_Y WALL_HEIGHT
#define GREEN_BASE_X 0


//Locations of all targets (in inches)
#define TARGET0X (62)
#define TARGET0Y (DIST_FROM_WALL + WALL_HEIGHT)
 
#define TARGET1X (86)
#define TARGET1Y (DIST_FROM_WALL + WALL_HEIGHT)

#define TARGET2X (DIST_FROM_WALL + WALL_WIDTH)
#define TARGET2Y (85)

#define TARGET3X (DIST_FROM_WALL + WALL_WIDTH)
#define TARGET3Y (61)

#define TARGET4X (DIST_FROM_WALL + WALL_WIDTH)
#define TARGET4Y (36)

#define TARGET5X (DIST_FROM_WALL + WALL_WIDTH)
#define TARGET5Y (12)

#define TARGET6X (86)
#define TARGET6Y (-DIST_FROM_WALL)

#define TARGET7X (62)
#define TARGET7Y (-DIST_FROM_WALL)

#define TARGET8X (36)
#define TARGET8Y (-DIST_FROM_WALL)

#define TARGET9X (12)
#define TARGET9Y (-DIST_FROM_WALL)

#define TARGET10X (-DIST_FROM_WALL)
#define TARGET10Y (12)

#define TARGET11X (-DIST_FROM_WALL)
#define TARGET11Y (36)

#define TARGET12X (-DIST_FROM_WALL)
#define TARGET12Y (61)

#define TARGET13X (-DIST_FROM_WALL)
#define TARGET13Y (85)

#define TARGET14X (12)
#define TARGET14Y (DIST_FROM_WALL + WALL_HEIGHT)

#define TARGET15X (36)
#define TARGET15Y (DIST_FROM_WALL + WALL_HEIGHT)


//Define a type that contains x and y coordinates
typedef struct
{
  float x;
  float y;
}Location;

//Public function prototypes
void InitConstants(void);
Location GetTargetLocation(char TargetNumber);