//DeadReckoning
//This function handles the motion awareness, based on encoder readings

//#define TEST
//#define TIMER_TEST
//#define Wheel_Speed_TEST

#include <math.h>


#include <mc9s12e128.h>     /* derivative information */
#include <S12E128bits.h> /* E128 bit definitions  */
#include <stdio.h>
#include <stdlib.h>

#include <hidef.h>      /* common defines and macros */
#include <ADS12.h> 
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */


#include "S12eVec.h"
#include "DeadReckoning.h"
#include "Movement.h"
#include "InitPorts.h"
#include "PWM.h"
#include "Timers.h"
#include "Constants.h"
#include "Sensors.h"


#define _Kp_R    1.5    
#define _Ki_R     .4   
#define _Kp_L    1.5   
#define _Ki_L     .4   
#define UPDATE_TIMER 187  //Updates at ~1kHz

static float Xpos=0, Ypos=0;      //Measures in inches
static int Angle=0, LeftTicks, RightTicks;  //Measures in ticks
static unsigned int LeftEncodePeriod, RightEncodePeriod;

static int LEFT_SPEED=0, RIGHT_SPEED=0;

static unsigned int j = 0;
static unsigned int k = 0;
float RCurrentSpeed, LCurrentSpeed, LInt, RInt, LError, RError; 
char LDutyCycle, RDutyCycle;

//////////////////////////////////////////////////////////////
//Left Wheel Encoder Read
void interrupt _Vec_tim1ch6 LeftEncode(void){
  //static int i_left=0;
  static unsigned int LastEdge;
  unsigned int NewEdge;

  NewEdge = TIM1_TC6;
  LeftEncodePeriod = NewEdge - LastEdge;
  
  //(void)printf("NewEdge: %u, LastEdge: %u, Period: %u \r\n", NewEdge, LastEdge, EncodePeriod );
  LastEdge = NewEdge;
  
   
  //Wheel Direction is inferred by direction motor is being driven
  //(our bot should never apply power in opposite direction)
  //and encoder readings are potentially comprised in event of collision anywars

  if(LeftForward){  //Moving forward
    //Mesaured in ticks (# of ticks / TicksPerRotation = 1 rotation)
    LeftTicks++;
  }
  else{  
    LeftTicks--;
  }
  
  TIM1_TFLG1 = _S12_C6F;   //Clear the flag
  
}


///////////////////////////////////////////////////////////////
//Right Wheel Encoder Read
void interrupt _Vec_tim1ch5 RightEncode(void){
  
  //static int i_right=0;
  static unsigned int LastEdge;
  unsigned int NewEdge;
  

  NewEdge = TIM1_TC5;
  RightEncodePeriod = NewEdge - LastEdge;
  
  LastEdge = NewEdge;

  
  if(RightForward){  //Moving forward
    //Mesaured in ticks (# of ticks / TicksPerRotation = 1 rotation)
    RightTicks++;
  }
  else{  
    RightTicks--;
  }
  
  TIM1_TFLG1 = _S12_C5F;   //Clear the flag
}






/////////////////////////////////////////////////////////////////
//PID on the wheels

//Performs internal positioning calculations for both left and right wheels
void interrupt _Vec_tim0ch4 Wheel_PID(void)
{

  static float LeftIntegral=0, RightIntegral=0;
  static float Last_RightTarget, Last_LeftTarget;
  
  float RightTarget_in, LeftTarget_in;  
  float CurrentSpeed, Error;
  int Period;
  float DutyCycle;
     
    TIM0_TC4 = TIM0_TCNT + UPDATE_TIMER;  //Reset Timer
    TIM0_TFLG1 = _S12_C4F;   //Clear the flag
    EnableInterrupts;   //We don't want to miss encoder ticks
      
    
   
  
  
  //Scale and convert target speeds  
  LeftTarget_in = LEFT_SPEED/100.0;
  RightTarget_in = RIGHT_SPEED/100.0;
  
  if( RightTarget_in == 0)
  {
    RightIntegral = 0;
  }
  
  if( LeftTarget_in == 0)
  {
    LeftIntegral = 0;
  }
  
  
  Last_RightTarget = RightTarget_in;
  Last_LeftTarget = LeftTarget_in;

  //////////////////////////////////////////////////
  //Start Calculations for the left side
  //////////////////////////////////////////////////
    Period = LeftEncodePeriod;
    //if(Period < 50)
    //  return;
    
    //35.28 converts speed to roughly Rotations per second 
    //(precise units unneccessary as long as they are internally consistent)
    CurrentSpeed = 35.28/(float)Period;   
    LCurrentSpeed =  CurrentSpeed;
    
    if (LeftTarget_in < 0)
    {
      LeftTarget_in = -LeftTarget_in;
      Error = LeftTarget_in - CurrentSpeed;
      LError = Error;
      LeftIntegral += Error;
      LInt = LeftIntegral;
      DutyCycle = (Error * _Kp_L + LeftIntegral * _Ki_L);

      if(DutyCycle > 100)
      {
        DutyCycle = 100;  //Duty cycle can't be more than max value
        LeftIntegral -= Error;  //Unwind the overshoot
        
        //Alerts use that threshold has been reached and physical limitations prevent proper operation
        //(void)printf("MAX HIT!!!!!!!!!!!!!!!!! MAX HIT!!!!!!!!!!!!!!!!!!");
      }
      
      if( DutyCycle < 0 )
      {
        DutyCycle= 0;  //Duty cycle can't be more than max value
        LeftIntegral -= Error;//Unwind the overshoot
      }
        
      LDutyCycle = (char)DutyCycle;
      LeftBack((char)DutyCycle);
    } 
    else
    { 
      Error = LeftTarget_in - CurrentSpeed;
      LError = Error;
      LeftIntegral += Error;
      LInt = LeftIntegral;
      DutyCycle = (Error * _Kp_L + LeftIntegral * _Ki_L);
        
     
      if(DutyCycle > 100)
      {
        DutyCycle = 100;  //Duty cycle can't be more than max value
        LeftIntegral -= Error;  //Unwind the overshoot    
              
        //Alerts use that threshold has been reached and physical limitations prevent proper operation
        //(void)printf("MAX HIT!!!!!!!!!!!!!!!!! MAX HIT!!!!!!!!!!!!!!!!!!");
      }
      
      if( DutyCycle < 0 )
      {
        DutyCycle= 0;  //Duty cycle can't be more than max value
        LeftIntegral -= Error;//Unwind the overshoot
      }
        
      LDutyCycle = (char)DutyCycle;
      LeftFor((char)DutyCycle);
    }
    
  //////////////////////////////////////////////////
  //Start Calculations for the right side
  //////////////////////////////////////////////////
    Period = RightEncodePeriod;
    
    //if(Period < 50)
    //  return;
    
    // 1000ms/s * ( (187tick/ms) / (12encode/rotation) ) / (ticks/encode)
    // = rotations/second
    //1000*187/12 = 15583.3
    CurrentSpeed = 35.28/(float)Period;    
    RCurrentSpeed =  CurrentSpeed;

    if (RightTarget_in < 0)
    {
      RightTarget_in = -RightTarget_in;
      Error = RightTarget_in - CurrentSpeed;
      RError = Error;
      RightIntegral += Error;
      RInt = RightIntegral;
      DutyCycle = (Error * _Kp_R + RightIntegral * _Ki_R);
        
      if(DutyCycle > 100)
      {
        DutyCycle = 100;  //Duty cycle can't be more than max value
        RightIntegral -= Error;  //Unwind the overshoot
      }
      
      if(DutyCycle < 0)
      {
        DutyCycle= 0;  //Duty cycle can't be more than max value
        RightIntegral -= Error;//Unwind the overshoot
      }
        
      RDutyCycle = (char)(DutyCycle);  
      RightBack((char)(DutyCycle));
    }
    else
    {
      Error = RightTarget_in - CurrentSpeed;
      RError = Error;
      RightIntegral += Error;
      RInt = RightIntegral;
      DutyCycle = (Error * _Kp_R + RightIntegral * _Ki_R);
        
      if(DutyCycle > 100)
      {
        DutyCycle = 100;  //Duty cycle can't be more than max value
        RightIntegral -= Error;  //Unwind the overshoot
      }
      
      if(DutyCycle < 0)
      {
        DutyCycle= 0;  //Duty cycle can't be more than max value
        RightIntegral -= Error;//Unwind the overshoot
      }
        
      RDutyCycle = (char)(DutyCycle);  
      RightFor((char)(DutyCycle));
    }
      
}








////////////////////////////////////////////////////////////////
//Calculates Distance Change
//void interrupt _Vec_tim0ch5 PositionControl(void){
//The event needs to process very quickly for accuracy (make it high on the priority list)
void PositionMath(void)
{
  static int OldLeftTicks, OldRightTicks;
  int NewLeftTicks, NewRightTicks, DeltaTicks, DeltaLeft, DeltaRight;
                   
  if((TIM0_TFLG1 & _S12_C5F) != 0){
    //(void)printf("inside if statement");
    TIM0_TC5 = TIM0_TCNT + UPDATE_TIMER;   //Update the timer        
    TIM0_TFLG1 = _S12_C5F;   //Clear the flag
   
    NewLeftTicks = LeftTicks;
    NewRightTicks = RightTicks;
   
    //Error due to small angle approx ~ (ticks/10000)^2 
    //for ticks = 1, sin(ticks)/ticks ~ 10^-8
    //for ticks = 100,  sin(ticks)/ticks ~ 10^-4
    DeltaLeft = (NewLeftTicks-OldLeftTicks);
    DeltaRight = (NewRightTicks - OldRightTicks);
    DeltaTicks = DeltaRight + DeltaLeft;

    Angle += DeltaRight - DeltaLeft;  //This is probably correct
    //Angle += DeltaLeft - DeltaRight;
    /*if(Angle > TicksPerRotation/2)
    {
      Angle = Angle - TicksPerRotation;
    }
    if(Angle < -TicksPerRotation/2)
    {
      Angle = Angle + TicksPerRotation;
    }
    printf ("Angle is %i\r\n", Angle);
    */
    
    //Divide by 2 accounts for 2 wheels moving in same direction 
    Xpos += DeltaTicks * DistancePerTick * cos(Angle*(2*PI/TicksPerRotation)) / 2;
    Ypos += DeltaTicks * DistancePerTick * sin(Angle*(2*PI/TicksPerRotation)) / 2;
    
         
    //(void)printf("NewLeft: %i  OldLeft: %i  NewRight: %i  OldRight: %i\n\r", NewLeftTicks, OldLeftTicks, NewRightTicks, OldRightTicks);
    //(void)printf("Angle: %6.5f, X-position: %6.3f, Y-position: %6.3f\n\r", (Angle*(2*PI/TicksPerRotation)),Xpos,Ypos);   
 
    OldLeftTicks = NewLeftTicks;
    OldRightTicks = NewRightTicks;
 
  }
}


///////////////////////////////////////////////////////////////
//External Data calls: since I have the interrupts in this file, 
//these can be used to access the variables instead of module lvl vars
//Interrupts disable around the variable call to avoid manipulating 'half of a float'

int GetAngle(void){
  int ReturnAngle;
  //DisableInterrupts;
  ReturnAngle = Angle;
  //EnableInterrupts;
 return ReturnAngle; 
}

void ReckonAngle(int NewAngle){
  Angle = NewAngle;
}

float GetXpos(void){
  float ReturnXpos;
  ReturnXpos = Xpos;
 return ReturnXpos; 
}



float GetYpos(void){
  float ReturnYpos;
  ReturnYpos = Ypos;
 return ReturnYpos; 
}

void ReckonXpos(float X){
  Xpos = X;
}

void ReckonYpos(float Y){
  Ypos = Y; 
}



////////////////////////////////////////////////////////////////////////
//Test Harnesses


#ifdef TIMER_TEST
void main(void)
{
  float LeftTarget = 0, RightTarget = 0;
  char KeyStroke;

  (void)printf("Starting");
  InitPorts();
  InitPWM();
  InitTimers();
  
  while(1)
  {
  
    if ( kbhit() != 0)
    {
      KeyStroke = getchar();
    
      switch (KeyStroke)
      {
        
        case '8':
          LeftTarget+=.1;
          RightTarget+=.1;
          break;
        case '2':
          LeftTarget-=.1;
          RightTarget-=.1;
          break;
        case '7':
          LeftTarget+=.1;
          break;
        case '1':
          LeftTarget-=.1;
          break;
        case '9':
          RightTarget+=.1;
          break;
        case '3':
          RightTarget-=.1;
          break;
      }
      //(void)printf("RightTarget: %f    LeftTarget:  %f", RightTarget, LeftTarget);
    }
    
    PositionMath();
    //WheelSpeed(LeftTarget, RightTarget);
    LeftSpeed(105);
    RightSpeed(75);
  }
  
}



#endif


//Was implemented as update for interrupt driven wheel PID
void WheelSpeed(int LeftTarget, int RightTarget)
{
  LEFT_SPEED = LeftTarget;
  RIGHT_SPEED = RightTarget;
}

#ifdef Wheel_Speed_TEST

void main(void)
{
  (void)printf("Starting");
  InitPorts();
  InitPWM();
  InitTimers();

  //LeftSpeed(0);
  //RightSpeed(50);
  //(void)printf("-passed left right speed-");
  

  while(1)
  {
  WheelSpeed(0,20);
  //(void)printf("Left: Target: %6.3f, CurrSpeed: %6.3f, Int: %6.3f  Diff: %6.3f  Duty: %i \n\r",LEFT_SPEED/100.0, LCurrentSpeed, (LInt * _Ki_L), (LError * _Kp_L), LDutyCycle); 
  (void)printf("Right: Target: %6.3f, CurrSpeed: %6.3f, Int: %6.3f  Diff: %6.3f  Duty: %i \n\r",RIGHT_SPEED/100.0, RCurrentSpeed, (RInt * _Ki_R), (RError * _Kp_R), RDutyCycle);      
  //printf("L %u\n\r", LeftTicks);
  //printf("R %u\n\r", RightTicks);
  //printf("trhough");  
  
  }
}

#endif