//DeadReckoning
//This function handles the motion awareness, based on encoder readings

//#define TEST
//#define TIMER_TEST
#define Wheel_Speed_TEST

#include <math.h>


#include <mc9s12e128.h>     /* derivative information */
#include <S12E128bits.h> /* E128 bit definitions  */
#include <stdio.h>
#include <stdlib.h>

#include <hidef.h>      /* common defines and macros */
#include <ADS12.h> 
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */


#include "S12eVec.h"
#include "DeadReckoning.h"
#include "Movement.h"
#include "InitPorts.h"
#include "PWM.h"
#include "Timers.h"
#include "Constants.h"
#include "Sensors.h"


#define _Kp_ 0.5     // 1
#define _Ki_ .1      //3
#define UPDATE_TIMER 500 //187  //Rough value; updates at ~1kHz

static float Xpos=0, Ypos=0;                   //Measures in inches
static int Angle=0, LeftTicks, RightTicks;  //Measures in ticks
static unsigned int LeftEncodePeriod, RightEncodePeriod;

static int LEFT_SPEED=0, RIGHT_SPEED=0;
//static int Thresh_Update = 200;

static unsigned int j = 0;
static unsigned int k = 0;
float RTargetShow, LTargetShow, RCurrentShow, LCurrentShow;
char RightDutyCycleShow, LeftDutyCycleShow;

void LeftUpdateControl(float TargetRPM, float CurrentRPM) 
{
  static float LeftIntegral; //sum error
  float LeftError;
  float LeftDutyCycle;

      /*if(!LeftForward)
        CurrentRPM = -CurrentRPM;
      */
      LeftError = TargetRPM - CurrentRPM;
      LeftIntegral += LeftError;
      LeftDutyCycle = (LeftError * _Kp_ + LeftIntegral * _Ki_);
        
     
      if(LeftDutyCycle > 100)
      {
        LeftDutyCycle = 100;  //Duty cycle can't be more than max value
        LeftIntegral -= LeftError;  //Unwind the overshoot
        //(void)printf("MAX HIT!!!!!!!!!!!!!!!!! MAX HIT!!!!!!!!!!!!!!!!!!");
      } 
      else if( LeftDutyCycle < -100 )
      {
        LeftDutyCycle= -100;  //Duty cycle can't be more than max value
        LeftIntegral -= LeftError;//Unwind the overshoot
      }
        
      LCurrentShow = CurrentRPM;
      LeftDutyCycleShow = LeftDutyCycle;
      LeftSpeed((char)LeftDutyCycle);
      
      //(void)printf("Left: Target: %6.3f, CurrSpeed: %6.3f, Int: %6.3f  Diff: %6.3f  Duty: %i \n\r", LeftTarget, CurrentSpeed, (LeftIntegral * _Ki_), (Error * _Kp_), DutyCycle); 
}

void RightUpdateControl(float TargetRPM, float CurrentRPM) 
{
  static float RightIntegral; //sum error
  float RightError;
  float RightDutyCycle;
  //RightUpdateFlag;

      /*if(!LeftForward)
        CurrentRPM = -CurrentRPM;
      */
      RightError = TargetRPM - CurrentRPM;
      RightIntegral += RightError;
      RightDutyCycle = (RightError * _Kp_ + RightIntegral * _Ki_);
        
     
      if(RightDutyCycle > 100)
      {
        RightDutyCycle = 100;  //Duty cycle can't be more than max value
        RightIntegral -= RightError;  //Unwind the overshoot
        //(void)printf("MAX HIT!!!!!!!!!!!!!!!!! MAX HIT!!!!!!!!!!!!!!!!!!");
      } 
      else if( RightDutyCycle < -100 )
      {
        RightDutyCycle= -100;  //Duty cycle can't be more than max value
        RightIntegral -= RightError;//Unwind the overshoot
      }
        
      RCurrentShow = CurrentRPM;
      RightDutyCycleShow = RightDutyCycle;
      
      RightSpeed((char)RightDutyCycle);
      //RightUpdateFlag
      //(void)printf("Left: Target: %6.3f, CurrSpeed: %6.3f, Int: %6.3f  Diff: %6.3f  Duty: %i \n\r", LeftTarget_in, CurrentSpeed, (LeftIntegral * _Ki_), (Error * _Kp_), DutyCycle); 
}



//////////////////////////////////////////////////////////////
//Left Wheel Encoder Read
void interrupt _Vec_tim1ch5 LeftEncode(void){
  //static int i_left=0;
  static unsigned int LastEdge;
  unsigned int NewEdge;

  j++;
  NewEdge = TIM1_TC5;
  LeftEncodePeriod = NewEdge - LastEdge;
  
  //(void)printf("NewEdge: %u, LastEdge: %u, Period: %u \r\n", NewEdge, LastEdge, EncodePeriod );
  LastEdge = NewEdge;
  
   
  //(void)printf("left");
  //Wheel Direction is inferred by direction motor is being driven
  //(our bot should never apply power in opposite direction)
  //and encoder readings are potentially comprised in event of collision anywars

  if(LeftForward){  //Moving forward
    //Mesaured in ticks (# of ticks / TicksPerRotation = 1 rotation)
    LeftTicks++;
  }
  else{  
    LeftTicks--;
  }
  
  TIM1_TFLG1 = _S12_C5F;   //Clear the flag
  
}


///////////////////////////////////////////////////////////////
//Right Wheel Encoder Read
void interrupt _Vec_tim1ch6 RightEncode(void){
  
  //static int i_right=0;
  static unsigned int LastEdge;
  unsigned int NewEdge;
  

  NewEdge = TIM1_TC6;
  RightEncodePeriod = NewEdge - LastEdge;
  
  LastEdge = NewEdge;

  
  if(RightForward){  //Moving forward
    //Mesaured in ticks (# of ticks / TicksPerRotation = 1 rotation)
    RightTicks++;
  }
  else{  
    RightTicks--;
  }
  
  TIM1_TFLG1 = _S12_C6F;   //Clear the flag
}






/////////////////////////////////////////////////////////////////
//PID on the wheels
//void WheelSpeed(int LeftTarget, int RightTarget)
void interrupt _Vec_tim0ch4 Wheel_PID(void)
{
  float RightTargetDesired, LeftTargetDesired;  
  float RightCurrentSpeed, LeftCurrentSpeed;
  int RightPeriod, LeftPeriod;

    TIM0_TFLG1 = _S12_C4F;   //Clear the flag
    TIM0_TC4 = TIM0_TCNT + UPDATE_TIMER;  //Reset Timer
    EnableInterrupts;   //We don't want to miss encoder ticks
      
    
    //Scale and convert target speeds  
    LeftTargetDesired = LEFT_SPEED/100.0;
    RightTargetDesired = RIGHT_SPEED/100.0;
    
    LeftPeriod = LeftEncodePeriod;
    RightPeriod = RightEncodePeriod;
    
    LeftCurrentSpeed = 35.28/(float)LeftPeriod;
    RightCurrentSpeed = 35.28/(float)RightPeriod;
  
    LeftUpdateControl(LeftTargetDesired, LeftCurrentSpeed);
    RightUpdateControl(RightTargetDesired, RightCurrentSpeed);
    //////////////////////////////////////////////////
    //Start Calculations for the left side
    //////////////////////////////////////////////////
      //(void)printf("Right:  Target: %6.3f, CurrentSpeed: %6.3f,  Duty: %i \n\r", RightTarget_in, CurrentSpeed, DutyCycle);
}



////////////////////////////////////////////////////////////////
//Calculates Distance Change
//void interrupt _Vec_tim0ch5 PositionControl(void){
//The event needs to process very quickly for accuracy (make it high on the priority list)
void PositionMath(void)
{
  static int OldLeftTicks, OldRightTicks;
  int NewLeftTicks, NewRightTicks, DeltaTicks, DeltaLeft, DeltaRight;
                   
  if((TIM0_TFLG1 & _S12_C5F) != 0){
    //(void)printf("inside if statement");
    TIM0_TC5 = TIM0_TCNT + UPDATE_TIMER;   //Update the timer        
    TIM0_TFLG1 = _S12_C5F;   //Clear the flag
   
    NewLeftTicks = LeftTicks;
    NewRightTicks = RightTicks;
   
    //Error due to small angle approx ~ (ticks/10000)^2 
    //for ticks = 1, sin(ticks)/ticks ~ 10^-8
    //for ticks = 100,  sin(ticks)/ticks ~ 10^-4
    DeltaLeft = (NewLeftTicks-OldLeftTicks);
    DeltaRight = (NewRightTicks - OldRightTicks);
    DeltaTicks = DeltaRight + DeltaLeft;

    Angle += DeltaLeft - DeltaRight;
    if(Angle > TicksPerRotation/2)
    {
      Angle = Angle - TicksPerRotation;
    }
    if(Angle < -TicksPerRotation/2)
    {
      Angle = Angle + TicksPerRotation;
    }
    
    //Divide by 2 accounts for 2 wheels moving in same direction 
    Xpos += DeltaTicks * DistancePerTick * cos(Angle*(2*PI/TicksPerRotation)) / 2;
    Ypos += DeltaTicks * DistancePerTick * sin(Angle*(2*PI/TicksPerRotation)) / 2;
    
         
    //(void)printf("NewLeft: %i  OldLeft: %i  NewRight: %i  OldRight: %i\n\r", NewLeftTicks, OldLeftTicks, NewRightTicks, OldRightTicks);
    //(void)printf("Angle: %6.5f, X-position: %6.3f, Y-position: %6.3f\n\r", (Angle*(2*PI/TicksPerRotation)),Xpos,Ypos);   
 
    OldLeftTicks = NewLeftTicks;
    OldRightTicks = NewRightTicks;
 
  }
}


///////////////////////////////////////////////////////////////
//External Data calls: since I have the interrupts in this file, 
//these can be used to access the variables instead of module lvl vars
//Interrupts disable around the variable call to avoid manipulating 'half of a float'

int GetAngle(void){
  int ReturnAngle;
  //DisableInterrupts;
  ReturnAngle = Angle;
  //EnableInterrupts;
 return ReturnAngle; 
}

void ReckonAngle(int NewAngle){
  Angle = NewAngle;
}

float GetXpos(void){
  float ReturnXpos;
  ReturnXpos = Xpos;
 return ReturnXpos; 
}



float GetYpos(void){
  float ReturnYpos;
  ReturnYpos = Ypos;
 return ReturnYpos; 
}

void ReckonXpos(float X){
  Xpos = X;
}

void ReckonYpos(float Y){
  Ypos = Y; 
}



////////////////////////////////////////////////////////////////////////
//Testing


#ifdef TIMER_TEST
void main(void)
{
  float LeftTarget = 0, RightTarget = 0;
  char KeyStroke;
  //int Ang, NewRightTicks, NewLeftTicks, LastRightTicks, LastLeftTicks, count;

  (void)printf("Starting");
  InitPorts();
  InitPWM();
  InitTimers();
  
  while(1)
  {
  
    if ( kbhit() != 0)
    {
      KeyStroke = getchar();
    
      switch (KeyStroke)
      {
        
        case '8':
          LeftTarget+=.1;
          RightTarget+=.1;
          break;
        case '2':
          LeftTarget-=.1;
          RightTarget-=.1;
          break;
        case '7':
          LeftTarget+=.1;
          break;
        case '1':
          LeftTarget-=.1;
          break;
        case '9':
          RightTarget+=.1;
          break;
        case '3':
          RightTarget-=.1;
          break;
      }
      (void)printf("RightTarget: %f    LeftTarget:  %f", RightTarget, LeftTarget);
    }
    
    PositionMath();
    //WheelSpeed(LeftTarget, RightTarget);
    LeftSpeed(105);
    RightSpeed(75);
  }
  
  
  //RightSpeed(100);
  //LeftSpeed(100);
  /*while(1)
  {
    NewLeftTicks = LeftTicks;
    NewRightTicks = RightTicks;
    if( (NewLeftTicks != LastLeftTicks)  || (NewRightTicks != LastRightTicks) ) {  
      count += 1;
      if(count == 10000){
        count = 0;
        LastRightTicks = NewRightTicks;
        LastLeftTicks = NewLeftTicks;
        PositionMath();  
        X = GetXpos();
        Y = GetYpos();
        Ang = GetAngle();
        (void)printf("Count: %i\r\n", count);
        (void)printf("Angle: %i, X-position: %6.2f, Y-position: %6.2f\n\r",Ang,X,Y);   
        (void)printf("Left: %i        Right     %i\n\r", NewLeftTicks, NewRightTicks);
      }
    }
  }*/
}



#endif


//Was implemented as update for interrupt driven wheel PID
void WheelSpeed(int LeftTarget, int RightTarget)
{
  LEFT_SPEED = LeftTarget;
  RIGHT_SPEED = RightTarget;
}

#ifdef Wheel_Speed_TEST

void main(void)
{
  (void)printf("Starting");
  InitPorts();
  InitPWM();
  InitTimers();

  //LeftSpeed(50);
  //RightSpeed(50);
  //(void)printf("-passed left right speed-");
  WheelSpeed(50,50);
  while(1)
  {
    //(void)printf("Left: Target: %6.3f, CurrSpeed: %6.3f, Int: %6.3f  Diff: %6.3f  Duty: %i \n\r", LeftTarget_in, CurrentSpeed, (LeftIntegral * _Ki_), (Error * _Kp_), DutyCycle); 
    printf("LTarget: %i, LCurr: %6.3f, LDC: %i, RTarget: %i, RCurr: %6.3f, RDC: %i\n\r", LEFT_SPEED, LCurrentShow, LeftDutyCycleShow, RIGHT_SPEED, RCurrentShow, RightDutyCycleShow);
  //printf("L %u\n\r", LeftTicks);
  //printf("R %u\n\r", RightTicks);
  //printf("trhough");  
  
  }
}

#endif