/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
//Library includes
#include <hidef.h>      /* common defines and macros */
#include <stdlib.h>
#include <stdio.h>

//Custom includes
#include "SMEVENTS.h"
#include "FiringSM.h"
#include "Gun.h"
#include "ChkEvents.h"    // to be able to call GetTarget()


/*---------------------------- Module Functions ---------------------------*/
static Event_t DuringFiringActive( Event_t Event);
static Event_t DuringFiringIdle( Event_t Event);


/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static FiringState CurrentState;
static char FireFlag;
static int MissedBall = 0;
/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   unsigned char: the event to process

 Returns
   unsigned char: an event to return

 Description
   Runs firing sequence, to release ball ( assuming gun wheel is already at speed)
   If time allows, implemented gaurd on the wheel speed
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM
   Trevor Clark       3/11/10, cleaned up code
****************************************************************************/
Event_t RunFiringSM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   FiringState NextState = CurrentState;
   
   if (CurrentState == FIRING_ACTIVE)
   printf("current state is FIRING ACTIVE\r\n");
   else
   printf("current state is FIRING IDLE\r\n");

   switch ( CurrentState )
   {
       case FIRING_ACTIVE :       // If current state is state Firing_active
         {
         DuringFiringActive(CurrentEvent);
         }
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_BALL_IS_RELEASED : //If event is event one
                  // Execute action function for state one : event one
                  NextState = FIRING_IDLE;//Decide what the next state will be
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
            }
         }
         
         break;
         
       case FIRING_IDLE :       // If current state is state one

         DuringFiringIdle(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_TARGET_MISSED : //If event is event target_missed
                  if (MissedBall < 1) // guard
                  {
                    MissedBall++;
                    NextState = FIRING_ACTIVE;//Decide what the next state will be
                    MakeTransition = TRUE; //mark that we are taking a transition
                  }
               break;

            }
         }
         break;

    }
    
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       RunFiringSM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       RunFiringSM(EV_ENTRY);  //   Execute entry function for new state
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Activates firing sequence
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartFiringSM ( void)
{
   CurrentState = FIRING_ACTIVE;
   // call the entry function (if any) for the ENTRY_STATE
   RunFiringSM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
FiringState QueryFiringSM ( void )
{
   return(CurrentState);
}

char QueryFireFlag ( void )
{
  
  static char OldFlag;
  char NewFlag, change=0;
  NewFlag = FireFlag;
  
  if(NewFlag == 0)
    if(OldFlag == 1)
      change=1;
  
  OldFlag = NewFlag;
  
  return change;
}

/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringFiringActive( Event_t Event)
{
    // process EV_ENTRY & EV_EXIT events
    if ( Event == EV_ENTRY)
    {
        Release_Ball();
        FireFlag = 1;
    }else if ( Event == EV_EXIT)
    {
      FireFlag = 0;  //Reset Flag
    }else
    // do the 'during' function for this state
    {
      //DO NOTHING
    }
    return(Event);
}

static Event_t DuringFiringIdle( Event_t Event)
{
    // process EV_ENTRY & EV_EXIT events
    if ( Event == EV_ENTRY)
    {
      //DO NOTHING...its idle after all
    }else if ( Event == EV_EXIT)
    {
      //DO NOTHING...its idle after all
    }else
    // do the 'during' function for this state
    {
      //DO NOTHING...its idle after all
    }
    return(Event);
}