/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/


#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */
#include <hidef.h>      /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <stdio.h>
#include <S12E128bits.h> /* E128 bit definitions  */
#include <hidef.h>      /* common defines and macros */
#include <stdlib.h>

#include "GamePlaySM.h"
#include "PlayingSM.h"
#include "Search_And_DestroySM.h"
#include "ReturnHomeSM.h"
#include "SMEVENTS.h"
#include "Movement.h"
#include "Gun.h"
#include "FiringSM.h"
#include "Constants.h"
#include "ChkEvents.h"
#include "ADS12.h"




#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/
// define constants for the states for this machine
// and any other local defines

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/
static Event_t DuringWaiting( Event_t Event);
static Event_t DuringPlaying( Event_t Event);
static Event_t DuringReturnHome( Event_t Event);
static Event_t DuringGameOver( Event_t Event);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static GamePlayState CurrentState;
static char WaitingFlag = 0;
static char PlayingFlag = 0;
static char ReturnHomeFlag = 0;
static char GameOverFlag = 0;
static char Side;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM
****************************************************************************/
Event_t RunGamePlaySM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   GamePlayState NextState = CurrentState;
   Event_t EntryEventKind = EV_ENTRY;

   switch ( CurrentState )
   {
       case Waiting :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         (void)DuringWaiting(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               //case EV_START_GAME : //If event is event one
               case EV_RECIEVED_TARGET : //If event is event one
                  // Execute action function for state one : event one
                  NextState = Playing;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  //CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
         
       case Playing :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //(void)printf("Current State is Playing");
         (void)DuringPlaying(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_RETURN_HOME : //If event is event one
                  // Execute action function for state one : event one
                  NextState = ReturnHome;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  break;
                  
               case EV_GAME_OVER : //If event is event one
                  // Execute action function for state one : event one
                  NextState = GameOver;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
            
       case ReturnHome :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //(void)printf("Current State is Return Home State");
         (void)DuringReturnHome(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_REACH_HOME : //If event is event one
                  // Execute action function for state one : event one
                  NextState = GameOver;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  //CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
         
       case GameOver :       // Black Hole!!!, do nothing
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //(void)printf("Current State is Game Over");
         (void)DuringGameOver(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            //Do nothing
            //Dead-End: Game is over
         }
         break;
      // repeat state pattern as required for other states
   }
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       (void)RunGamePlaySM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       (void)RunGamePlaySM(EntryEventKind);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartGamePlaySM ( void)
{


   (void)printf("StartGamePlaySM\n\r");
        //CurrentState = Playing;
        CurrentState = Waiting;
        
   // call the entry function (if any) for the ENTRY_STATE
   (void)RunGamePlaySM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
GamePlayState QueryGamePlaySM ( void )
{
   return(CurrentState);
}

char QueryWaitingFlag ( void )
{
    return(WaitingFlag);
}

char QueryReturnHomeFlag ( void )
{
    return(ReturnHomeFlag);
}

char GetSide( void )
{
  return Side;
}
/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringWaiting( Event_t Event)
{

    char Target;
    
    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    {
        (void)printf("Current State is Waiting");
    }
    else if ( Event == EV_EXIT )
    { 
      
      Target = GetTarget();
      
      switch (Target)
      { 
        case 8:
          Side = 8;
          PTIAD = PTIAD | BIT4HI;
          (void)printf("Side 8, Green\n\r");
          break;
        case 0:
          Side = 0;
          PTIAD = PTIAD & BIT4LO;
          (void)printf("Side 0, Red\n\r");
          break;
        default:
          (void)printf("Error in determining side.");  
      }
      
    }else
    // do the 'during' function for this state
    {
    
    
    
        //DO NOTHING
    }
    return(Event);
}

static Event_t DuringPlaying( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    //if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    {
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        StartPlayingSM();
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
    }
    else if ( Event == EV_EXIT )
    {
        // on exit, give the lower levels a chance to clean up first
        //NewEvent RunLowerLevelSM(Event);
        
        //RunPlayingSM(Event);
        
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        
        
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(Event);
        
        RunPlayingSM(Event);
        
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}

static Event_t DuringReturnHome( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    //if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    {
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        StartReturnHomeSM();
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
    }
    else if ( Event == EV_EXIT )
    {
        // on exit, give the lower levels a chance to clean up first
        //(void)RunReturnHomeSM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(Event);
        (void)RunReturnHomeSM(Event);
        //ReturnHomeFlag = ReturnHome();
        
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}

static Event_t DuringGameOver( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    //if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    {
      
      //LeftSpeed(0);
      //RightSpeed(0);
      
      
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        //StartLowerLevelSM( Event );
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
    }
    else if ( Event == EV_EXIT )
    {
        // on exit, give the lower levels a chance to clean up first
        //NewEvent RunLowerLevelSM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(Event);
        //ReturnHomeFlag = ReturnHome();
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}