//#define Test_GunControl

#include <hidef.h>      /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <stdio.h>
#include <ADS12.h> 
#include <stdlib.h>
#include <S12E128bits.h> /* E128 bit definitions  */
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */
#include <stdlib.h>

#include "S12eVec.h"

//Custom module includes
#include "Timers.h"
#include "PWM.h"
#include "InitPorts.h"
#include "Gun_PID.h"
#include "Gun.h"
#include "DeadReckoning.h"
#include <math.h>
#include <timers12.h>
#include "ChkEvents.h"

#include "constants.h"

#define _ms_ *187   //187.5 cycles per ms for PWM_AS = (24million cycles/s) / (128)
#define UPDATE_PERIOD (187)
#define DISPLAY_PERIOD (1_ms_)

#define _Ki_ 0.01   
#define _Kp_ 2

#define FULL_GUN_DUTY 200
#define MIN_GUN_DUTY 50
                 
static unsigned int EncodePeriod;
static int Target_Speed;  
//float Target;  

static float IntErr, SpeedTargetShow; 
int PeriodShow;
unsigned char DutyCy; 
static float CurrSpeed;
int debugFlag = 0;

static unsigned int j = 0;

//Counts Encoder Ticks
void interrupt _Vec_tim1ch4 Gun_Encoder(void)
{
  static unsigned int LastEdge;
  unsigned int NewEdge;

  NewEdge = TIM1_TC4;
  EncodePeriod = NewEdge - LastEdge;
  
  LastEdge = NewEdge;
  
    //EnableInterrupts;
  TIM1_TFLG1 = _S12_C4F;   //Clear the flag

} 

//PID on the wheels
//void GunSpeed(int Target)
void interrupt _Vec_tim0ch6 Gun_SpeedControl(void)
{
  static float Integral=0;
  float Speed, Error;
  int Period;
  float DutyCycle;

     j++;
  
  //if((TIM0_TFLG1 & _S12_C6F) != 0)
  //{
    
    TIM0_TC6 = TIM0_TCNT + DISPLAY_PERIOD;  //Reset Timer
    TIM0_TFLG1 = _S12_C6F;   //Clear the flag
    EnableInterrupts;   //We don't want to miss encoder ticks
    

    Period = EncodePeriod;
    
      // 1000ms/s * ( (187tick/ms) / (12encode/rotation) ) / (ticks/encode)
      // = rotations/second
      //1000*187/12 = 15583.3
      Speed = 15583.3/(float)Period;    
      SpeedTargetShow = Target_Speed;
      Error = (float)Target_Speed - Speed;
      //SpeedTargetShow = Target;
      //Error = (float)Target - Speed;
      Integral += Error;
      DutyCycle = (Error * _Kp_ + Integral * _Ki_);
        
      if(DutyCycle > FULL_GUN_DUTY)
      {
        DutyCycle = FULL_GUN_DUTY;  //Duty cycle can't be more than max value
        Integral -= Error;  //Unwind the overshoot
      }
      //if((DutyCycle < MIN_GUN_DUTY))
      if((DutyCycle < 0))
      {
        DutyCycle = 0;  //Duty cycle can't be more than max value
        Integral -= Error;//Unwind the overshoot
      }
        
      PWMDTY2 = (char)DutyCycle;
      
      // For printf
        PeriodShow = Period;
        IntErr = Integral;
        DutyCy = (char)DutyCycle;
        CurrSpeed = Speed;  
}


void GunSpeed(int Target)
{
  Target_Speed = Target;
}

int GetGunSpeed( void )
{
    
  int TargetAngle, Speed;
  float DeltaX, DeltaY, Distance;
  Location TargetLocation;
  //TC_Answer_t CurrentTarget;
  int CurrentTarget;
  
  CurrentTarget = GetTarget();
  //TargetLocation = GetTargetLocation(CurrentTarget.ActiveNum);  
  TargetLocation = GetTargetLocation(CurrentTarget);
   
  DeltaX = TargetLocation.x - GetXpos();
  DeltaY = TargetLocation.y - GetYpos();
  
  Distance = sqrt(DeltaX*DeltaX + DeltaY*DeltaY);
  
  //Map Speed
 
  if(CurrentTarget == 2 || CurrentTarget == 5 || CurrentTarget == 10 || CurrentTarget == 13)
  {
    //6 inch targets
    //The 6 inch targeting was never mapped, but the few required values were tested emperically
    if(Distance < 50)
      return 30;
    
    //if its above all of those
      return 40;
  }
  else //12 inch targets
  {
  	if(Distance  < 36)
        return 56;
  	if(Distance  < 38)
        return 54;
    if(Distance  < 40)
        return 52;
    if(Distance  < 42)
        return 49;
    if(Distance < 44)
      return 48;
    if(Distance  < 50)
        return 47;
    if(Distance  < 60)
        return 49;
    if(Distance  < 62)
        return 48;
    if(Distance  < 66)
        return 47;
    if(Distance  < 68)
        return 46;
    if(Distance  < 70)
        return 47;
    if(Distance  < 74)
        return 48;
    if(Distance  < 78)
        return 50;
  
    //anything higher gets this value
    return 54;
  }
  
}



/////////////////////////////////////////////////////////////////////////////////////
//Test Harness
#ifdef Test_GunControl

void main(void)
{
  //char KeyStroke;
  (void)printf("Test Fire!!!\n\r");
  InitPorts();
  InitTimers();
  InitPWM();
  //PWMDTY2 = MIN_GUN_DUTY;
  GunSpeed(50);
  //WheelSpeed(0,0);
  
  while (1)
  {
    //GunSpeed(0);
    //GunSpeed(50);
    (void)printf("NTargetSp: %6.3f, CurrSp: %6.3f, Period: %i, Int Err:  %6.3f, Duty: %i,  Count: %i \n\r", SpeedTargetShow, CurrSpeed, PeriodShow, IntErr, DutyCy, j);
  }
}

#endif