//Movement

//#define TEST
//#define TEST2
//#define TEST3
//#define TEST4


/////////////////////////////////////////////////////////////////////
//Functions built to be inserted into loops.
//Return status=1 for functions that are still active
//IMPLEMENTATION:
//When the function is complete, gaurd against calling it again
//
//
/////////////////////////////////////////////////////////////////////




#include <hidef.h>      /* common defines and macros */
#include <mc9s12e128.h>     /* derivative information */
#include <stdio.h>
#include <ADS12.h> 
#include <stdlib.h>
#include <S12E128bits.h> /* E128 bit definitions  */
#include <bitdefs.h> /* BIT0HI, BIT0LO....definitions */
#include <stdlib.h>
#include <math.h>

#include "DeadReckoning.h"
#include "Sensors.h"
#include "Movement.h"

#include "Timers.h"
#include "PWM.h"
#include "InitPorts.h"
#include "Constants.h"
#include "SPI.h"
#include "ChkEvents.h"

#define THRESH_ANGLE 10   //Angle is how close the bot needs to be to stop correcting b/c its 'close enough'
#define PROXY_THRESH_POS .1
#define PID_THRESH_POS 1


#define K_INT_POS 0.00005 //Integral co-efficient for PID position adjust
#define K_DELTA_POS .0002 //Delta co-efficient for PID position adujst

#define K_INT_ANG .00000005 //Integral co-efficient for PID angle adjust
#define K_DELTA_ANG .001 //Delta co-efficient for PID angle adjust

void LeftFor(char);
void RightFor(char);
void LeftBack(char);
void RightBack(char);
void LeftStop(void);
void RightStop(void);


//******************Begin Module code****************************


//Input: The direction you want to face
//Return: Status marker (1 is still rotating, 0 if done)
char SettingAngle(int TargetAngle){
  static float  Integral=0;
  
  int CurrentAngle;
  float Delta;
  float Speed;
  
  CurrentAngle = GetAngle();
  Delta = (TargetAngle - CurrentAngle);
  
  if(Delta > THRESH_ANGLE)
  {
    //Slow rotation speed to avoid overshoot: Angles are very sensitive
    WheelSpeed(-10, 10);
    //(void)printf("greater than thresh\r\n");
    return 1;
  }
  
  if(Delta < -THRESH_ANGLE)
  {  
    //Slow rotation speed to avoid overshoot: Angles are very sensitive
    WheelSpeed(10, -10);
    //(void)printf("less than thresh\r\n");
    return 1;
  }
  
  WheelSpeed(0,0);
  
  (void)printf("Reached Thresh\r\n");
  (void)printf("DELTA: %6.4f\n\r", Delta);
  return 0;
 
}

char RotateToBeacon() 
{
    PTIAD &= BIT4LO;
    LeftSpeed(-97);
    RightSpeed(100);
    if((PTIAD & BIT5HI) == 1) 
    {
        PTIAD |= BIT4HI; // Light LED
        LeftSpeed(0);
        RightSpeed(0);
        return 1;  
    }
    else 
    {
        return 0;
    }
}


////////////////////////////////////////////////////////////////////
//Tape Following
//Puts you at a certain PID controlled position on the line of tape
char DrivingTape(float Speed)
{
  char static ResetReckoning;
  int Tape1,Tape2,Tape3,     Tape7,Tape8,Tape9;


    //FORWARDS (It only drives forward now)
    Tape1 = CheckTape(1); //Right side
    Tape3 = CheckTape(3); //Left Side
    
   if(Tape1 == 0 && Tape3 == 0 ) //We go straight on the line
    {
      WheelSpeed(30,30);
      
      //(void)printf("\r\nStraight");
      return 1;
    }
  
    if(Tape1 == 1 && Tape3 == 0 ) //If Right ones on only
    {
      //Veer Right
      WheelSpeed(30,29); //30, 20
      
      //(void)printf("\r\nTurn Right");
      ResetReckoning = 1;
      return 1;
    }
    
    if(Tape1 == 0 && Tape3 == 1 ) //If Left ones on only
    {
      //Veer Left
      WheelSpeed(29,30);  //20, 30
      
      //(void)printf("\r\n           Turn Left");
      ResetReckoning = 1;
      return 1;
    }  
    
}
  
/////////////////////////////////////////////////////////////////
//Returns Angle You should Rotate to aim correctly

//Uses some trig to and notions of bot location and target locations in order to calculate the angle
//that bot needs to face (used for Rotating to Target and Home)

//Although angles are returned in encoder ticks, they are internally dealt with in radians 
//for  compatability with C library function
int CalculateTargetAngle(void)
{
  float ArcTan, ArcSin, RadsToTicks;
  int TargetAngle;
  float DeltaX, DeltaY, Distance, Correction;
  Location TargetLocation;
  //TC_Answer_t CurrentTarget;
  int CurrentTarget;

  CurrentTarget = GetTarget();
  TargetLocation = GetTargetLocation(CurrentTarget);


  (void)printf("Target#: %i, TargetX:%6.4f  TargetY:%6.4f  CurrentX:%6.4f  CurrentY:%6.4f  ", CurrentTarget, TargetLocation.x, TargetLocation.y, GetXpos(), GetYpos());

   
  DeltaX = TargetLocation.x - GetXpos();
  DeltaY = TargetLocation.y - GetYpos();
  
  (void)printf("deltax = %f   deltay = %f\r\n", DeltaX, DeltaY);
  
  Distance = sqrt(DeltaX*DeltaX + DeltaY*DeltaY);
  
  (void)printf("Inside Asin: %f\n\r", (Distance/GUN_TO_CENTER) );
  
  //Get Angle (assuming quadrants 1 or IV)
  ArcTan =atan(DeltaY/DeltaX); //This atan(2/24) is an imperical correction term (presumed cause - Tape following or gun error)
  
  //Despite our best efforts to have all mathematics generally applicable, in practice there were small,
  //but consistent errors, which we dealt with via these small correction terms
  switch (CurrentTarget)
  {
    case 0:
      Correction = -atan(2.0/24.0);
      break;
    case 1:
      Correction = -atan(1.5/28);
      break;
    case 2:
      Correction = 0;
      break;
    case 3:
      Correction = 0;
      break;
    case 4:
      Correction = -atan(1/60);
      break;
    case 5:
      Correction = 0;
      break;
    case 6:
      Correction = 0;
      break;
    case 7:
      Correction = +atan(2.0/75.0);
      break;
  }
  
  ArcTan = ArcTan + Correction;
  
  
  ArcSin = asin(GUN_TO_CENTER/Distance);  //This is a correction term, since the gun is not in the bot center
  RadsToTicks = TicksPerRotation / (2*PI);


  //Targets 16 & 17 are the Green and Red bases, however they were included in the Targets list 
  //so that this function could be simply applied for the return home state
  if(CurrentTarget == 16  || CurrentTarget == 17)
  {
    TargetAngle = (int)(ArcTan*RadsToTicks);
    (void)printf("Calculating Home Angle...Before phase correction: %6.4f\n\r",(TargetAngle/RadsToTicks) ); 
  }
  else
  {
    TargetAngle =  (int)( (ArcTan - ArcSin)*RadsToTicks ) - TicksPerRotation/4;
    (void)printf("Calculating Target Angle...Before phase correction: %6.4f\n\r",(TargetAngle*360.0/TicksPerRotation) ); 
    
  }


  //If angle is in quadrants II or III, we need to correct by a phase factor
  if(DeltaX < 0){ 
    if(DeltaY > 0){
      //TargetAngle += (float)TicksPerRotation/2;  //Quadrant II
      TargetAngle = TicksPerRotation/2 + TargetAngle;  //Quadrant II
    }
    else{
      
      //TargetAngle -= (float)TicksPerRotation/2;  //Quadrant III
      TargetAngle = -TicksPerRotation/2 + TargetAngle;  //Quadrant III
    }
  }
  
  (void)printf("ArcTan: %6.4f,   ArcSin: %6.4f,  TargetAngle: %6.4f\n\r", ArcTan,ArcSin,(TargetAngle*(360.0/TicksPerRotation) ) );

  return TargetAngle;
}
  
  
////////////////////////////////////////////////////////////////  
//Setting Wheel Speeds  
//Dont use this function except inside PID
//This is a direct PWM control
  
void LeftSpeed(char Duty)
{
  if(Duty > 0)
  {
    LeftFor(Duty);
  }
  if(Duty < 0)
  {
    LeftBack((char)abs(Duty));
  }
  if(Duty == 0)
  {
    LeftStop();
  }
}

void RightSpeed(char Duty)
{
  if(Duty > 0){
    RightFor(Duty);
  }
  if(Duty < 0)
  {
    RightBack((char)abs(Duty));
  }
  if(Duty == 0)
  {
    RightStop();
  }
}


//////////////////////////////////////////////////////////////////
//Internal Functions

//These functions are written seperate from LeftSpeed and RightSpeed because of the added complexity
//of setting polarity and the directional bit


void RightFor(char Duty){
  //(void)printf("->Right Forward; Duty: %i\n\r", Duty);
  PWMPOL |= _S12_PPOL0; //Sets polarity high duty cycle for PWM1
  PWMDTY0 = (char)((FULL_DUTY*Duty)/100);
  PTP = PTP & BIT2LO;   //Direction to Forward
}          
void RightBack(char Duty){
  //(void)printf("->Right Backward; Duty: %i\n\r", Duty);
  PWMPOL &= ~_S12_PPOL0;   //Sets polarity low cycle for PWM1
  PWMDTY0 = (char)(FULL_DUTY*Duty/100); 
  PTP = PTP | BIT2HI;     //Direction to Reverse
  
}
void RightStop(void)
{
  
  //(void)printf("->Right Stop\n\r");
  PWMDTY0 = 0;
  //Stop Right Wheel
}


void LeftBack(char Duty){ 
  //(void)printf("->Left Backward; Duty: %i\n\r", Duty);
  PWMPOL |= _S12_PPOL1;   //Sets polarity high duty cycle for PWM0
  PWMDTY1 = (char)((FULL_DUTY*(Duty))/100);
  PTP = PTP & BIT1LO;     //Sets Direction Forward

}
void LeftFor(char Duty){
  //(void)printf("->Left Forward; Duty: %i\n\r", Duty);
  PWMPOL &= ~_S12_PPOL1;   //Sets polarity low duty cycle for PWM0
  PWMDTY1 = (char)((FULL_DUTY*Duty)/100);
  PTP = PTP | BIT1HI;     //Sets direction to Reverse
}
void LeftStop(void)
{
  //(void)printf("->Left Stop\n\r");
  PWMDTY1 = 0;
}


/////////////////////////////Test Harnesses//////////////////////////////////


#ifdef TEST
void main(void)
{
  float Ang,X=0,Y=0;
  int KeyStroke, TargetAngle=(TicksPerRotation/1.1);
  char Active =1, Reset=1;
  
  InitPorts();
  InitPWM();
  InitTimers();
  
  ReckonAngle(0);
  ReckonXpos(0);
  ReckonYpos(0);
        
  (void)printf("Starting\n\r");
  (void)printf("Starting Position: %f\r\n", GetXpos());
  (void)printf("K_INT_ANG: %f    K_DELTA_ANG:  %f\n\r", K_INT_ANG, K_DELTA_ANG);
            
  //while(SettingAngle(45*TicksPerRotation) != 0)
  while(1)
  {
  
  SettingAngle(TicksPerRotation/2);
    PositionMath();
    //SettingAngle(90);
    //DrivingTape();
  }
}
#endif


  /*
  //Remote Control
    if ( kbhit() != 0){
    
      KeyStroke = getchar();
      switch (KeyStroke)
      {
        case 'r':  //Reset
        (void)printf("Resetting Angle");
          Active = 1;
          break;
        case '8':  //Forward
          (void)printf("Moving Forward\n\r");
          RightSpeed(100);
          LeftSpeed(100);
          break;
        case '2':  //Backward
          (void)printf("Moving Backward\n\r");
          RightSpeed(-100);
          LeftSpeed(-100);
          break;
        case '4':  //Left
          (void)printf("Rotating Left\n\r");
          RightSpeed(100);
          LeftSpeed(-100);
          break;
        case '6':   //Right
          (void)printf("Rotating Right\n\r");
          RightSpeed(-100);
          LeftSpeed(100);
          break;
        case '5':   //Stop
          (void)printf("Stopped\n\r");
          RightSpeed(0);
          LeftSpeed(0);
          break;
        case 'u':   //Update
          (void)printf("Position update:\n\r");
          Ang = GetAngle();
          X = GetXpos();
          Y = GetYpos();
          (void)printf("Angle: %f, X-position: %f, Y-position: %f\n\r",Ang,X,Y);
          break;
        case '9':   //Right Wheel Forward Only
          (void)printf("Right Wheel Forward Only\n\r");
          RightSpeed(100);
          LeftSpeed(0);
          break;
        case '7':   //Left Wheel Forward Only
          (void)printf("Left Wheel Forward Only\n\r");
          RightSpeed(0);
          LeftSpeed(100);
          break;
        case '3':   //Right Wheel Backward Only
          (void)printf("Right Wheel Backward Only\n\r");
          RightSpeed(-100);
          LeftSpeed(0);
          break;
        case '1':   //Left Wheel backward Only
          (void)printf("Left Wheel backward Only\n\r");
          RightSpeed(0);
          LeftSpeed(-100);
          break;
        case '+': //increase cannon power
          break;
        case '-': //decrease cannon power
          break;
        case 'f':  //fire
          break;
      }  //end switch
    }//end if
    
  //} //end while
//} //End testing main
*/


#ifdef TEST2
void main(void)
{
  
  
  InitPorts();
  InitPWM();
  InitTimers();
  
  (void)printf("Sensor:  1  2  3  5  7  8  9\n\r");  
  //LeftSpeed(100);
  //RightSpeed(100);
  while(1)
  {
  (void)printf("\r\n Value:  %i  %i  %i  %i  %i  %i  %i", CheckTape(1), CheckTape(2), CheckTape(3), CheckTape(5), CheckTape(7), CheckTape(8), CheckTape(9));
    DrivingTape(1);
  }
}
 #endif
 
#ifdef TEST3
void main(void)
{
  
  
  InitPorts();
  InitPWM();
  InitTimers();
  
  (void)printf("Sensor:  1  2  3  5  7  8  9\n\r");  
  PWMDTY1 = 60;
  PWMDTY2 = 60;
  while(1)
  {
  //(void)printf("\r\n Value:  %i  %i  %i  %i  %i  %i  %i", CheckTape(1), CheckTape(2), CheckTape(3), CheckTape(5), CheckTape(7), CheckTape(8), CheckTape(9));
  //  DrivingTape(1);
  }
}
 #endif
 
 
 #ifdef TEST4
 
 void main(void)
 {
  float Ang,X=0,Y=0;
  int KeyStroke, TargetAngle=(TicksPerRotation/1.1);
  char Active =1, Reset=1;
  int angle;
  
  InitConstants();
  
  ReckonAngle(0);
  ReckonXpos(0);
  ReckonYpos(0);
  
  angle = CalculateTargetAngle();
  
  printf("the Angle is: %i ", angle);
 
 }
 #endif