/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "PlayingSM.h"
#include "Search_And_DestroySM.h"
#include "ReturnHomeSM.h"

#include <hidef.h>      /* common defines and macros */
#include <stdlib.h>
#include "SMEVENTS.h"
#include "Movement.h"
#include "Gun.h"
#include "Gun_PID.h"
#include "FiringSM.h"
#include "Sensors.h"
#include "DeadReckoning.h"
#include "Constants.h"
#include "GamePlaySM.h"

#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/
// define constants for the states for this machine
// and any other local defines

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/
static Event_t DuringFindTape( Event_t Event);
static Event_t DuringAlignTape( Event_t Event);
static Event_t DuringFindTIntersect( Event_t Event);
static Event_t DuringSearchAndDestroy( Event_t Event);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static PlayingState CurrentState;
static char TapeFoundFlag = 0;
static char TapeAlignedFlag = 0;
static char TIntersectFlag = 0;
static char SearchAndDestroyFlag = 0;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM
   Trevor Clark,      3/11/10  
****************************************************************************/
Event_t RunPlayingSM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   PlayingState NextState = CurrentState;
   Event_t EntryEventKind = EV_ENTRY;

   switch ( CurrentState )
   {
       case FindTape :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //NewEvent = DuringFindTape(CurrentEvent);
         DuringFindTape(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_TAPE_FOUND : //If event is event one
                  // Execute action function for state one : event one
                  NextState = AlignTape;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
       case AlignTape :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //NewEvent = DuringAlignTape(CurrentEvent);
         DuringAlignTape(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_TAPE_ALIGNED : //If event is event one
                  // Execute action function for state one : event one
                  NextState = FindTIntersect;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
       case FindTIntersect :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //NewEvent = DuringFindTIntersect(CurrentEvent);
         DuringFindTIntersect(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_TIntersect_FOUND : //If event is event one
                  // Execute action function for state one : event one
                  WheelSpeed(0,0);
                  NextState = SearchAndDestroy;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
       case SearchAndDestroy :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //NewEvent = DuringFindTIntersect(CurrentEvent);
         DuringSearchAndDestroy(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_RECIEVED_TARGET : //If event is event one
                  // Execute action function for state one : event one
                  NextState = SearchAndDestroy;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  break;

                // repeat cases as required for relevant events
            }
         }
         break;
      // repeat state pattern as required for other states
    }
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       (void)RunPlayingSM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       (void)RunPlayingSM(EntryEventKind);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartPlayingSM ( void )
{
   // to implement entry to a history state or directly to a substate
   // you can modify the initialization of the CurrentState variable
   // otherwise just start in the entry state every time the state machine
   // is started
   //if ( EV_ENTRY_HISTORY != CurrentEvent )
   {
        CurrentState = FindTape;
   }
   // call the entry function (if any) for the ENTRY_STATE
   (void)RunPlayingSM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
PlayingState QueryPlayingSM ( void )
{
   return(CurrentState);
}


char QueryTapeFoundFlag(void)
{
  static char OldTape = 0;
  char NewTape, change = 0;
    
  NewTape = CheckTape(5);
    
  if(OldTape == 0)
    if(NewTape == 1)
      change = 1;
      
  OldTape = NewTape;
      
  return change;
}

char QueryTapeAlignedFlag ( void )
{
  /*static char OldTape = 0;
  char NewTape, change = 0;
    
  NewTape = CheckTape(1);
  //NewTape = CheckTape(7);
    
  if(OldTape == 1)
    if(NewTape == 0)
      change = 1;
      
  OldTape = NewTape;*/
  static char OldTape = 0;
  char NewTape, change = 0;
    
  NewTape = CheckTape(2);
  //NewTape = CheckTape(7);
    
  if(OldTape == 0)
    if(NewTape == 1)
      change = 1;
      
  OldTape = NewTape;
      
  return change;
}

char QueryTIntersectFlag ( void )
{
  static char OldTape = 0;
  char NewTape, change = 0;
    
  NewTape = (CheckTape(1) && CheckTape(3) );
  //NewTape = (CheckTape(9) && CheckTape(7) );
    
    
  if(OldTape == 0)
    if(NewTape == 1)
      change = 1;
      
  OldTape = NewTape;
  
  return change;
}

/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringFindTape( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    //if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    if ( (Event == EV_ENTRY) )
    {
      (void)printf("Entering FindTape\n\r");
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        //StartLowerLevelSM( Event );
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
        WheelSpeed(32,40);    //17, 21  //65, 75

    }
    else if ( Event == EV_EXIT )
    {
      (void)printf("State is FindingTape");
        TapeFoundFlag = 0;
        WheelSpeed(0,0);
        // on exit, give the lower levels a chance to clean up first
        //NewEvent RunLowerLevelSM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        //RightSpeed(75);
        //LeftSpeed(50);
        //(void)printf("Start the wheels\n\r");
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(Event);
        //TapeFoundFlag = FindTape();
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}

static Event_t DuringAlignTape( Event_t Event)
{
    //Event_t NewEvent;

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    //if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    if ( (Event == EV_ENTRY) )
    {
    (void)printf("Entering AlignTape\n\r");
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        //StartLowerLevelSM( Event );
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
        WheelSpeed(12,-12);
    }
    else if ( Event == EV_EXIT )
    {
        WheelSpeed(0,0);
        
        TapeAlignedFlag = 0;
        // on exit, give the lower levels a chance to clean up first
        //NewEvent RunLowerLevelSM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
      //Do Nothing - PID interrupts shouold be moving us right now
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}

static Event_t DuringFindTIntersect( Event_t Event)
{

    if ( (Event == EV_ENTRY) )
    {
    (void)printf("Entering FindTIntersect\n\r");
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        //StartLowerLevelSM( Event );
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
        
    }
    else if ( Event == EV_EXIT )
    {
        (void)printf("Exiting Find Intersect\n\r");
        
        if(GetSide() != 0)
        {
          ReckonXpos(TAPE_T_GREEN_X);   //Sets internal understanding of current location
          ReckonYpos(TAPE_T_GREEN_Y+4.3);   //Sets internal understanding of current location
          ReckonAngle(TAPE_T_GREEN_ANGLE);  //Sets internal understanding of current angle
          //ReckonAngle(0);
        }
        else 
        {
          ReckonXpos(TAPE_T_RED_X);   //Sets internal understanding of current location
          ReckonYpos(TAPE_T_RED_Y-4.3);   //Sets internal understanding of current location
          ReckonAngle(TAPE_T_RED_ANGLE);  //Sets internal understanding of current angle
          //ReckonAngle(TicksPerRotation/2);
        }
        
        WheelSpeed(0,0);
        TIntersectFlag = 0;
        // on exit, give the lower levels a chance to clean up first
        //NewEvent RunLowerLevelSM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        GunSpeed(GetGunSpeed()); 
        DrivingTape(.85);
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(Event);
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}

static Event_t DuringSearchAndDestroy( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    //if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    if ( (Event == EV_ENTRY) )
    {
      (void)printf("Entering SearchAndDestroy\n\r");
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        StartSearchAndDestroySM(  );
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
    }
    else if ( Event == EV_EXIT )
    {
      
        // on exit, give the lower levels a chance to clean up first
        (void)RunSearchAndDestroySM(Event);
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        (void)RunSearchAndDestroySM(Event);
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}
