/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "GamePlaySM.h"
#include "PlayingSM.h"
#include "Search_And_DestroySM.h"
#include "ReturnHomeSM.h"

#include <hidef.h>      /* common defines and macros */
#include <stdlib.h>
#include "SMEVENTS.h"
#include "Movement.h"
#include "Gun.h"
#include "FiringSM.h"
#include "Constants.h"
#include "DeadReckoning.h"
#include "Sensors.h"

#include <stdio.h>

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/

static Event_t DuringRotateHome(Event_t Event);
static Event_t DuringGoHome(Event_t Event);
static Event_t DuringAlignTapeHome( Event_t Event);
static Event_t DuringFindTIntersectHome( Event_t Event);
static Event_t DuringCoastHome( Event_t Event);
static Event_t DuringGameOver_HOME( Event_t Event);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static ReturnHomeState CurrentState;
static char AngleHomeFlag=1;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM
****************************************************************************/
Event_t RunReturnHomeSM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   ReturnHomeState NextState = CurrentState;
   Event_t EntryEventKind = EV_ENTRY;

   switch ( CurrentState )
   {
        
        case RotateHome :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringRotateHome(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_ROTATE_HOME : //If event is event one
                  // Execute action function for state one : event one
                  NextState = GoHome;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  //CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;  
        
       case GoHome :       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringGoHome(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_GOT_HOME : //If event is event one
                  // Execute action function for state one : event one
                  NextState = GameOver_Home;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  //CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break; 
        
        
       case GameOver_Home :       // Black Hole!!!, do nothing
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         (void)printf("Current State is Game Over Home");
         DuringGameOver_HOME(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            //Game is over...Lets hang out for all eternity and do nothing!!!
         }
         
       break;
      // repeat state pattern as required for other states
    }
    
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       (void)RunReturnHomeSM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       (void)RunReturnHomeSM(EntryEventKind);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartReturnHomeSM ( void)
{

   (void)printf("StartReturnHomeSM\n\r");
   // to implement entry to a history state or directly to a substate
   // you can modify the initialization of the CurrentState variable
   // otherwise just start in the entry state every time the state machine
   // is started
   //if ( EV_ENTRY_HISTORY != CurrentEvent )
   {
   
        CurrentState = RotateHome;
        //CurrentState = Waiting;
   }
   // call the entry function (if any) for the ENTRY_STATE
   (void)RunReturnHomeSM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
char QueryAngleHomeFlag ( void )
{
  static char OldFlag=1;
  char NewFlag, change=0;
  NewFlag = AngleHomeFlag;
  //(void)printf("OldFlag: %i  Newflag is %i\r\n", OldFlag, NewFlag);
  if((NewFlag == 0) && (OldFlag == 1))
  {  
    change=1;
  }
  
  OldFlag = NewFlag;
  if(change == 1)
    (void)printf("Query Angle Home Flag Change: %i", change);
  return change;
}


//Check Front center tape sensor to see if we're aligned
char QueryTapeAlignedHomeFlag ( void )
{
  static char OldTape = 0;
  char NewTape, change = 0;
    
  NewTape = CheckTape(1);
    
  if(OldTape == 1)
    if(NewTape == 0)
      change = 1;
      
  OldTape = NewTape;
      
  return change;
}


//Uses front-left and front-right tape sensors to tell if we have reached end of tape
char QueryTIntersectHomeFlag ( void )
{
  static char OldTape = 0;
  char NewTape, change = 0;
    
  NewTape = (CheckTape(1) && CheckTape(3) );
    
    
  if(OldTape == 0)
    if(NewTape == 1)
      change = 1;
      
  OldTape = NewTape;
  
  return change;
}

//For event checker...Checks Dead Reckoing to see if we've reached home
char QueryReachHomeFlag ( void )
{
  static char OldStatus = 0;
  char NewStatus, change = 0, NearRedBase, NearGreenBase;
    
    
  NearRedBase = ( (GetXpos() > (WALL_WIDTH - 8))  && (GetYpos() < 8) );
  NearGreenBase = ( (GetXpos() < 8)  && (GetYpos() > (WALL_HEIGHT - 8)) );
  if(  NearRedBase ||  NearGreenBase )
    NewStatus = 1;
  else
    NewStatus = 0;
    
    
  if(OldStatus == 0)
    if(NewStatus == 1)
      change = 1;
      
  OldStatus = NewStatus;
      
  return change;
}

/***************************************************************************
 private functions
 ***************************************************************************/


static Event_t DuringRotateHome( Event_t Event)
{
  static int TargetAngle;
    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    {
        //Get TargetAngle
        TargetAngle = CalculateTargetAngle();
        (void) printf ("Entering DuringRotateHome\r\n");
        (void)printf("Target Angle: %f   CurrentAngle:  %f", (TargetAngle*(360.0/TicksPerRotation) ), (GetAngle()*(360.0/TicksPerRotation)) );
        GunSpeed(0);
        //DO NOTHING
    }
    else if ( Event == EV_EXIT )
    {
      (void) printf ("Exiting DuringRotateHome\r\n");
      WheelSpeed(0,0);

    }else
    // do the 'during' function for this state
    {
        //Keep aadjusting until you reach appropriate angle
        AngleHomeFlag = SettingAngle(TargetAngle);
    }
    return(Event);
}


static Event_t DuringGoHome( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    {
        (void) printf ("Entering DuringGoHome\r\n");
        //DO NOTHING
    }
    else if ( Event == EV_EXIT )
    {
      WheelSpeed(0,0);

    }else
    // do the 'during' function for this state
    {
    
        WheelSpeed(50,50);     // Rotate to find tape
    }
    return(Event);
}



static Event_t DuringGameOver_HOME( Event_t Event)
{
    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) )
    {
      (void)printf("Entering DuringGameOver_HOME");
    }
    else if ( Event == EV_EXIT )
    {
      //Do nothing
    }else
    {
       //Do Nothing
    }
    // Do not remap event
    return(Event);
}
