/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "SPI_On_SM.h"
#include "SPI.h"
#include <mc9s12e128.h>     /* derivative information */
#include <S12E128bits.h> /* E128 bit definitions  */
#include <stdio.h>

#include "Timers.h"


/*----------------------------- Module Defines ----------------------------*/
// define constants for the states for this machine
// and any other local defines

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/
static Event_t DuringQueryForActiveTarget( Event_t Event);
static Event_t DuringEnableIRBeacon( Event_t Event);
static Event_t DuringReportBeaconStatus( Event_t Event);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static SPI_On_State CurrentState;
static unsigned int CurrentTarget;
static unsigned int TargetCount = 0;
static unsigned char CommandResponse;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM 
****************************************************************************/
Event_t RunSPI_On_SM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   SPI_On_State NextState = CurrentState;
   Event_t EntryEventKind = EV_ENTRY;

   switch ( CurrentState )
   {
       case QueryForActiveTarget:       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringQueryForActiveTarget(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_OFFLINE : //If event is event one
                  // Execute action function for state one : event one
                  printf("offline\r\n");
                  NextState = QueryForActiveTarget;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //??????EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  //????CurrentEvent = NewEvent;
                  break;
               
               case EV_GAME_NOT_STARTED : //If event is event one
                  // Execute action function for state one : event one
                  printf("delayed game\r\n");
                  NextState = QueryForActiveTarget;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                
               case EV_RECIEVED_TARGET : //If event is event one
                  // Execute action function for state one : event one
                  printf("receive target\r\n");
                  printf("Prepare to proceed to enableIR\r\n");
                  NextState = EnableIRBeacon;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                  
               case EV_COMMANDER_BUSY:
                  // Execute action function for state one : event one
                  printf("busy\r\n");
                  NextState = ReportBeaconStatus;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                  
            }
         }
         break;
         
       case EnableIRBeacon:       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringEnableIRBeacon(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_COMMANDER_BUSY : //If event is event one
                  // Execute action function for state one : event one
                  printf("busy\r\n");
                  NextState = EnableIRBeacon;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                  
               case EV_BEACON_ACTIVE_NOT_ILLUM : //If event is event one
                  // Execute action function for state one : event one
                  printf("active, not illu\r\n");
                  NextState = EnableIRBeacon;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                                    
               case EV_BEACON_ACTIVE_ILLUM : //If event is event one
                  // Execute action function for state one : event one
                  printf("active, illu\r\n");
                  NextState = ReportBeaconStatus;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
                  
              case EV_BEACON_HIT:
                  // Execute action function for state one : event one
                  printf("hit!\r\n");
                  NextState = QueryForActiveTarget;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
              /*case WAITING_FOR_STATUS:
                  // Execute action function for state one : event one
                  NextState = QueryForActiveTarget;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;  */
            }
         }
         break;
         
       case ReportBeaconStatus:       // If current state is state one
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         DuringReportBeaconStatus(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_COMMANDER_BUSY:
                  // Execute action function for state one : event one
                  printf("busy\r\n");
                  NextState = ReportBeaconStatus;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;

               case EV_BEACON_ACTIVE_NOT_ILLUM: 
                  // Execute action function for state one : event one
                  printf("active, not illu\r\n");
                  NextState = ReportBeaconStatus;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
          
              case EV_BEACON_ACTIVE_ILLUM: 
                  // Execute action function for state one : event one
                  printf("active, illu\r\n");
                  NextState = ReportBeaconStatus;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
              case EV_BEACON_HIT:
                  // Execute action function for state one : event one
                  printf("hit!\r\n");
                  NextState = QueryForActiveTarget;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break;
              /*case WAITING_FOR_STATUS: 
                  // Execute action function for state one : event one
                  NextState = ReportBeaconStatus;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  break; */
           
            }
         }
         
  }
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       (void)RunSPI_On_SM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       (void)RunSPI_On_SM(EntryEventKind);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartSPI_On_SM ()
{
   // to implement entry to a history state or directly to a substate
   // you can modify the initialization of the CurrentState variable
   // otherwise just start in the entry state every time the state machine
   // is started
   //if ( EV_ENTRY_HISTORY != CurrentEvent )
   //{
        CurrentState = QueryForActiveTarget;
   //}
   // call the entry function (if any) for the ENTRY_STATE
   (void)RunSPI_On_SM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
SPI_On_State QuerySPI_On_State ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringQueryForActiveTarget( Event_t Event)
{
    

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if (Event == EV_ENTRY)
    {
       
       printf(" %x \r\n ", CommandResponse);
       printf("In QueryForActiveTarget_ State\n\r");
    }
    else if ( Event == EV_EXIT )
    {
        //CurrentTarget = CommandResponse - 0xC0;
       CurrentTarget = (unsigned int)(CommandResponse & 0b00001111); 
       //CurrentTarget = (CommandResponse & 0b00001111); 
       
       printf("The current Target is: %u\r\n", CurrentTarget);

    }else
    // do the 'during' function for this state
    {
        
         // Query
      if(SPI_TIMER_EXPIRE)
      {
        SetSPITimer(200);
        CommandResponse = Request_IR_Illumination(GetTarget());
      }
       /*CommandResponse = Query_Current_Target();
       printf(" %i \r\n ", CommandResponse);
       printf("In QueryForActiveTarget_ State during\n\r");*/
       
    }
        
        
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}



static Event_t DuringEnableIRBeacon( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( Event == EV_ENTRY)
    {
      printf("In EnableIRBeacon_ State\n\r");
    }
    else if ( Event == EV_EXIT )
    {
        //DO NOTHING

    }else
    // do the 'during' function for this state
    { 
      // Query
      if(SPI_TIMER_EXPIRE)
      {
        SetSPITimer(200);
        CommandResponse = Request_IR_Illumination(GetTarget());
      }
        /*CommandResponse = Request_IR_Illumination(GetTarget());
        printf("In EnableIRBeacon_ State during\n\r");*/
    }
    
    
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}



static Event_t DuringReportBeaconStatus( Event_t Event)
{


    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( Event == EV_ENTRY )
    {
        printf("In ReportBeaconStatus_ State\n\r");
    }
    else if ( Event == EV_EXIT )
    {
        //DO NOTHING

    }else
    {
        // Query
      if(SPI_TIMER_EXPIRE)
      {
        SetSPITimer(200);
        CommandResponse = Request_Beacon_Status(GetTarget());
      }
        /*CommandResponse = Request_IR_Illumination(GetTarget());
        printf("In ReportBeaconStatus_ State during\n\r");   */
    }
    
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}


unsigned int GetTarget(void)
{
  return CurrentTarget;
}

unsigned char GetCommandResponse(void)
{
    return CommandResponse;
}