/****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "Search_And_DestroySM.h"

#include <hidef.h>      /* common defines and macros */
#include <stdlib.h>
#include "SMEVENTS.h"
#include "Movement.h"
#include "Gun.h"
#include "FiringSM.h"
#include "Constants.h"
#include "DeadReckoning.h"
#include <stdio.h>


/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/
static Event_t DuringRotateToShoot( Event_t Event);
static Event_t DuringFire1( Event_t Event);
//static Event_t DuringFire2( Event_t Event);
//static Event_t DuringSearchAndDestroyIdle( Event_t Event);

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static SearchAndDestroyState CurrentState;
static char AngleFlag=1;
//static char FireFlag;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunSearchAndDestroySM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM
****************************************************************************/
Event_t RunSearchAndDestroySM( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* are we making a state transition? */
   SearchAndDestroyState NextState = CurrentState;
   //  Event_t NewEvent;  //  ????  What does this do.  Does this do history??
   Event_t EntryEventKind = EV_ENTRY;

   switch ( CurrentState )
   {
       case RotateToShoot :       // If current state is RotateToShoot
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //NewEvent = DuringStateOne(CurrentEvent);  prob dont need this
         DuringRotateToShoot(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_ROTATE_DONE : //If event is Rotate_Done
                  
                  DuringRotateToShoot(CurrentEvent);
                  // Execute action function for state one : event one
                  NextState = Fire1;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  // // EntryEventKind - EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  // // CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
       case Fire1 :       // If current state is Fire1
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         //NewEvent = DuringStateOne(CurrentEvent);  prob dont need this
         DuringFire1(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            //Local Dead-End, exit via higher level state change
         }
         break;

      // repeat state pattern as required for other states
    }
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       RunSearchAndDestroySM(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       (void)RunSearchAndDestroySM(EntryEventKind);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartSearchAndDestroySM (void)
{
  //No history

        CurrentState = RotateToShoot;
   // call the entry function (if any) for the ENTRY_STATE
   RunSearchAndDestroySM(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
SearchAndDestroyState QuerySearchAndDestroySM ( void )
{
   return(CurrentState);
}


/****************************************************************************
 Function
     QueryAngleFlag

 Parameters
     None

 Returns
     true false flag from testing for expired state on timer

 Description
     returns the current state of the oven timer expired flag
 Notes

 Author
     J. Edward Carryer, 2/21/05, 4:02PM
****************************************************************************/
char QueryAngleFlag ( void )
{
  static char OldFlag;
  char NewFlag, change=0;
  NewFlag = AngleFlag;
  
  if(NewFlag == 0 && OldFlag == 1)
    change=1;
  
  OldFlag = NewFlag;
  
  return change;
}


/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringRotateToShoot( Event_t Event)
{
    Event_t NewEvent;
    static signed int TargetAngle;

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    // // if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    if ( Event == EV_ENTRY)
    {
      (void)printf("RotateToShoot\n\r");
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        // // StartLowerLevelSM( Event );  // We dont have any
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
        //GunSpeed(50);
        
        //  RotateToTargetFAKE();  // We really dont have an entry function
        
        //Get TargetAngle
        TargetAngle = CalculateTargetAngle();
        (void)printf("Target Angle: %f   CurrentAngle:  %f", (TargetAngle*(360.0/TicksPerRotation) ), (GetAngle()*(360.0/TicksPerRotation)) );
        
        //Set the speed once you reach firing location; 
        //allowing time for the gun speed to adjust
        GunSpeed(GetGunSpeed()); 
        //TargetAngle = TicksPerRotation/5;
    }
    else if ( Event == EV_EXIT )
    {
        // on exit, give the lower levels a chance to clean up first
        //Stop Wheels
        WheelSpeed(0,0);
        
    }else
    // do the 'during' function for this state
    {
        AngleFlag = SettingAngle(TargetAngle);
       
        
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}


static Event_t DuringFire1( Event_t Event)
{

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    // // if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    if ( Event == EV_ENTRY)
    {
      (void)printf("Fire");
        // implement any entry actions required for this state machine
        // after that start any lower level machines that run in this state
        // only call Start if the lower level machine does not have a history
        StartFiringSM( );  
        // repeat the StartxxxSM() functions for concurrent state machines
        // on the lower level
    }
    else if ( Event == EV_EXIT )
    {
        // on exit, give the lower levels a chance to clean up first
        // repeat for any concurrently running state machines
        // now do any local exit functionality
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        //FireFlag = Release_Ball2();  
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
        
     //GunSpeed(50); 

    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}
