 
 /****************************************************************************
 Module
   d:\me218bTNG\Lectures\Lecture31\SMTemplate.c

 Revision
   1.0.1

 Description
   This is a template file for implementing state machines.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 02/13/10 14:29 jec      revised Start and run to add new kind of entry function
                         to make implementing history entry cleaner
 02/13/10 12:29 jec      added NewEvent local variable to During function and
                         comments about using either it or Event as the return
 02/11/10 15:54 jec      more revised comments, removing last comment in during
                         function that belongs in the run function
 02/09/10 17:21 jec      updated comments about internal transitions on During funtion
 02/18/09 10:14 jec      removed redundant call to RunLowerlevelSM in EV_Entry
                         processing in During function
 02/20/07 21:37 jec      converted to use enumerated type for events & states
 02/13/05 19:38 jec      added support for self-transitions, reworked
                         to eliminate repeated transition code
 02/11/05 16:54 jec      converted to implment hierarchy explicitly
 02/25/03 10:32 jec      converted to take a passed event parameter
 02/18/99 10:19 jec      built template from MasterMachine.c
 02/14/99 10:34 jec      Began Coding
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include <hidef.h>      /* common defines and macros */
#include <stdlib.h>

#include "StateMachine.h"
#include "SMEVENTS.h"



/*----------------------------- Module Defines ----------------------------*/
// define constants for the states for this machine
// and any other local defines



/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine, things like during
   functions, entry & exit functions.They should be functions relevant to the
   behavior of this state machine
*/

static Event_t DuringFindingTape( Event_t Event);
static Event_t DuringFollowingTape( Event_t Event);
Event_t DuringFiring( Event_t CurrentEvent);

//static Event_t DuringStateOne( Event_t Event)

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well
static State CurrentState;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
    RunTemplateSM

 Parameters
   Event_t: the event to process

 Returns
   Event_t: an event to return

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 2/11/05, 10:45AM
****************************************************************************/
Event_t RunStateMachine( Event_t CurrentEvent )
{
   unsigned char MakeTransition = FALSE;/* Assume we aren't making a transition */
   State NextState = CurrentState;  //Assume we keep the same state (b/c no transition)
   Event_t NewEvent;
   Event_t EntryEvent = EV_ENTRY;

   switch ( CurrentState )
   {
       case FindingTape :       // If current state is state FindingTape
         // Execute During function for state one. EV_ENTRY & EV_EXIT are
         // processed here
         NewEvent = DuringFindingTape(CurrentEvent);
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_TAPE7 : //If TAPE7 is detected
                  // Execute action function for state one : event one
                  NextState = FollowingTape;//Decide what the next state will be
                  
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  //EntryEventKind - EV_ENTRY_HISTORY;
                  break;
                // repeat cases as required for relevant events
            }
         }
         
       case FollowingTape:       // If current state is TapeFollow
         
         NewEvent = DuringFollowingTape(CurrentEvent);
         
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_TAPE9 : //If event is EV_TAPE9
                  // Execute action function for state one : event one
                  NextState = CurrentState;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = FALSE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  EntryEvent = EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
         
         
       case Firing:       // If current state is TapeFollow
         
         NewEvent = DuringFiring(CurrentEvent);
         
         //process any events
         if ( CurrentEvent != EV_NO_EVENT )        //If an event is active
         {
            switch (CurrentEvent)
            {
               case EV_HIT : //If event is EV_FIRED
                  // Execute action function for state one : event one
                  NextState = FollowingTape;//Decide what the next state will be
                  // for internal transitions, skip changing MakeTransition
                  MakeTransition = TRUE; //mark that we are taking a transition
                  // if transitioning to a state with history change kind of entry
                  EntryEvent = EV_ENTRY_HISTORY;
                  // optionally, consume or re-map this event for the upper
                  // level state machine
                  CurrentEvent = NewEvent;
                  break;
                // repeat cases as required for relevant events
            }
         }
         break;
      // repeat state pattern as required for other states
    }
    //   If we are making a state transition
    if (MakeTransition == TRUE)
    {
       //   Execute exit function for current state
       RunStateMachine(EV_EXIT);
       CurrentState = NextState; //Modify state variable
       //   Execute entry function for new state
       RunStateMachine(EntryEvent);
     }
     return(CurrentEvent);
}
/****************************************************************************
 Function
     StartTemplateSM

 Parameters
     None

 Returns
     None

 Description
     Does any required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 2/18/99, 10:38AM
****************************************************************************/
void StartStateMachine ( Event_t CurrentEvent )
{
   // to implement entry to a history state or directly to a substate
   // you can modify the initialization of the CurrentState variable
   // otherwise just start in the entry state every time the state machine
   // is started
   if ( EV_ENTRY_HISTORY != CurrentEvent )
   {
        CurrentState = ENTRY_STATE;
   }
   // call the entry function (if any) for the ENTRY_STATE
   RunStateMachine(EV_ENTRY);
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 2/11/05, 10:38AM
****************************************************************************/
State QueryStateMachine ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

static Event_t DuringFindingTape( Event_t CurrentEvent)
{
    Event_t NewEvent;

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (CurrentEvent == EV_ENTRY) || (CurrentEvent == EV_ENTRY_HISTORY) )
    {
      //Do nothing
    }
    else if ( CurrentEvent == EV_EXIT )
    {
      // now do any local exit functionality
      
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(CurrentEvent);
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(CurrentEvent);
}

static Event_t DuringFollowingTape( Event_t Event)
{
    Event_t NewEvent;

    // process EV_ENTRY, EV_ENTRY_HISTORY & EV_EXIT events
    if ( (Event == EV_ENTRY) || (Event == EV_ENTRY_HISTORY) )
    {
      //Do nothing
    }
    else if ( Event == EV_EXIT )
    {
      // now do any local exit functionality
      
    }else
    // do the 'during' function for this state
    {
        // run any lower level state machine
        //NewEvent = RunLowerLevelSM(Event);
        // repeat for any concurrent lower level machines
        // do any activity that is repeated as long as we are in this state
    }
    // return either Event, if you don't want to allow the lower level machine
    // to remap the current event, or NewEvent if you do want to allow it.
    return(Event);
}

