//Initializes Timer subsystem (rather than having inputs)
//I've just included a sample of working functionality,
//allowing for maximal customization

//#define Test

#include <hidef.h>      // common defines and macros 
#include <mc9s12e128.h>     // derivative information 
#include <stdio.h>  
//#include "s12eVec.h"   //?????
#include <stdlib.h>
#include <S12E128bits.h> // E128 bit definitions  
#include <bitdefs.h> // BIT0HI, BIT0LO....definitions 
#include <stdio.h>

#include "Timers.h"


#define _ms_ *(187)   //187.5 cycles per ms for PWM_AS = (24million cycles/s) / (128)
#define PERIOD (1_ms_)  //187*10 < 65535   :)




//Initializes two input capture(for IR sensor and Tape sensor compares,
//and an output compare (used in SetTimer)
void InitTimers(void){
   (void)printf("Initializing Timers\n\r");


   //********************Timer 0 - internal timers********************************
   TIM0_TSCR1 = _S12_TEN; //Turn Timer 0 system on
   TIM0_TSCR2 |= (_S12_PR0|_S12_PR1|_S12_PR2);  //set pre-scale to 128 in this case   
   
   //*******Set up Timer: Output Compare on Tim0:4 -> PT T0 (no connect)*******
   //Wheel Speed PID
   TIM0_TIOS |= _S12_IOS4;    //Sets to output Compare
   TIM0_TCTL1 = TIM0_TCTL1 & ~(_S12_OM4 | _S12_OL4); //No pin connected to TIM1_PT4, just an internal timer
   TIM0_TC4 = TIM0_TCNT + PERIOD; //The next timer will go off in TIMER_PERIOD ticks
   TIM0_TFLG1 = _S12_C4F; //Clear the flag
   TIM0_TIE |= _S12_C4I;
  
   //*******Set up Timer: Output Compare on Tim0:5 -> PT T1 (no connect)*******
   //Internal Timer for performing positioning mathematics
   TIM0_TIOS |= _S12_IOS5;    //Sets to output Compare
   TIM0_TCTL1 = TIM0_TCTL1 & ~(_S12_OM5 | _S12_OL5); //No pin connected to TIM1_PT4, just an internal timer
   TIM0_TC5 = TIM0_TCNT + PERIOD; //The next timer will go off in TIMER_PERIOD ticks
   TIM0_TFLG1 = _S12_C5F; //Clear the flag
   //TIM0_TIE |= _S12_C5I;  //Enable interrupt on timer1, line 5

   //*******Set up Timer: Output Compare on Tim0:6 -> PT T2 (no connect)*******
   //Internal Timer for gun Speed PID
   TIM0_TIOS |= _S12_IOS6;    //Sets to output Compare
   TIM0_TCTL1 = TIM0_TCTL1 & ~(_S12_OM6 | _S12_OL6); //No pin connected to TIM1_PT4, just an internal timer
   TIM0_TC6 = TIM0_TCNT + PERIOD; //The next timer will go off in TIMER_PERIOD ticks
   TIM0_TFLG1 = _S12_C6F; //Clear the flag 
   TIM0_TIE |= _S12_C6I;  // Enable Interrupt
    
   
   
  
   //********************Timer 1********************************
   TIM1_TSCR1 = _S12_TEN; //Turn Timer system on
   TIM1_TSCR2 |= (_S12_PR0|_S12_PR1|_S12_PR2);  //set pre-scale to 128 in this case   
   
   //*********Set up Input Capture on Tim1:4 -> PT T4***********
   //Captures signals from Gun Wheel Encoder
   TIM1_TIOS &= ~_S12_IOS4;  //set T4 to input capture
   TIM1_TCTL3 |= (_S12_EDG4A);  //Sets Mode for PT4 to capture rising edge  
   TIM1_TFLG1 = _S12_C4F;  //Clear flag 1 by setting it
   TIM1_TIE |= _S12_C4I;  //Enables Timer interrupt on line 4 
   
   //*********Set up Input Capture on Tim1:5 -> PT T5*********** 
   //Captures signals from an Left Wheel Encoder
   TIM1_TIOS &= ~_S12_IOS5;  //set T5 to input capture
   TIM1_TCTL3 |= (_S12_EDG5A);  //Sets Mode for PT5 to capture rising edge  
   TIM1_TFLG1 = _S12_C5F;  //Clear flag 1 by setting it
   TIM1_TIE |= _S12_C5I;  //Enables Timer interrupt on line 5   
   
   //*********Set up Input Capture on Tim1:6 -> PT T6***********
   //Captures signals from Right Wheel Encoder
   TIM1_TIOS &= ~_S12_IOS6;  //set T6 to input capture
   TIM1_TCTL3 |= (_S12_EDG6A);  //Sets Mode for PT6 to capture rising edge  
   TIM1_TFLG1 = _S12_C6F;  //Clear flag 1 by setting it
   TIM1_TIE |= _S12_C6I;  //Enables Timer interrupt on line 6 
   
   //********************Timer 2********************************
   TIM2_TSCR1 = _S12_TEN; //Turn Timer 2 system on
   TIM2_TSCR2 |= (_S12_PR0|_S12_PR1|_S12_PR2);  //set pre-scale to 128 in this case
   
   
   //GUN PID PWM
   TIM2_TIOS |= _S12_IOS4;    //Sets to output Compare
   TIM2_TCTL1 = TIM2_TCTL1 & ~(_S12_OM4 | _S12_OL4); //No pin connected to TIM1_PT4, just an internal timer
   TIM2_TC4 = TIM2_TCNT + PERIOD; //The next timer will go off in TIMER_PERIOD ticks
   TIM2_TFLG1 = _S12_C4F; //Clear the flag   
   
   //SPI Timer
   TIM2_TIOS |= _S12_IOS5;    //Sets to output Compare
   TIM2_TCTL1 = TIM2_TCTL1 & ~(_S12_OM5 | _S12_OL5); //No pin connected to TIM1_PT4, just an internal timer
   TIM2_TC5 = TIM2_TCNT + 10*PERIOD; //The next timer will go off in TIMER_PERIOD ticks
   TIM2_TFLG1 = _S12_C5F; //Clear the flag   

   TIM2_TIOS |= _S12_IOS6;    //Sets to output Compare
   TIM2_TCTL1 = TIM2_TCTL1 & ~(_S12_OM6 | _S12_OL6); //No pin connected to TIM1_PT4, just an internal timer
   TIM2_TC6 = TIM2_TCNT + 10*PERIOD; //The next timer will go off in TIMER_PERIOD ticks
   TIM2_TFLG1 = _S12_C6F; //Clear the flag 
   
   EnableInterrupts;  //Globally enables interrupts
   
}



//Input a time in ms (between 1 and 349).  This begin a counter which sets flag4 on Timer1
void SetTimer(unsigned int Time)
{ 
  TIM2_TC4 = TIM2_TCNT + (Time _ms_);
  TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag
}

void SetSPITimer(unsigned int Time)
{ 
  TIM2_TC5 = TIM2_TCNT + (Time _ms_);
  TIM2_TFLG1 = _S12_C5F; //Clear the OC2 flag
}



/////////////////////////Test Harness//////////////////////////////
#ifdef Test
void main(void)
{
  (void)printf("Starting");
  InitTimers();
  //SetTimer(100);
  while(1)
  {
    if((TIM2_TFLG1 & _S12_C4F) != 0 )
    {
      TIM2_TC4 = TIM2_TCNT + (100 _ms_);
      TIM2_TFLG1 = _S12_C4F; //Clear the OC2 flag
      (void)printf("10Hz\n\r");  
    }
  }
}
#endif